@@ -20,20 +20,20 @@
 
 /**
  * SECTION:element-dvdec
+ * @title: dvdec
  *
  * dvdec decodes DV video into raw video. The element expects a full DV frame
  * as input, which is 120000 bytes for NTSC and 144000 for PAL video.
  *
  * This element can perform simple frame dropping with the #GstDVDec:drop-factor
- * property. Setting this property to a value N > 1 will only decode every 
+ * property. Setting this property to a value N > 1 will only decode every
  * Nth frame.
  *
- * <refsect2>
- * <title>Example launch line</title>
+ * ## Example launch line
  * |[
  * gst-launch-1.0 filesrc location=test.dv ! dvdemux name=demux ! dvdec ! xvimagesink
  * ]| This pipeline decodes and renders the raw DV stream to a videosink.
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -45,6 +45,7 @@
 #include <gst/video/gstvideometa.h>
 #include <gst/video/gstvideopool.h>
 
+#include "gstdvelements.h"
 #include "gstdvdec.h"
 
 /* sizes of one input buffer */
@@ -58,15 +59,15 @@
 #define PAL_FRAMERATE_NUMERATOR 25
 #define PAL_FRAMERATE_DENOMINATOR 1
 
-#define PAL_NORMAL_PAR_X        59
-#define PAL_NORMAL_PAR_Y        54
-#define PAL_WIDE_PAR_X          118
-#define PAL_WIDE_PAR_Y          81
+#define PAL_NORMAL_PAR_X        16
+#define PAL_NORMAL_PAR_Y        15
+#define PAL_WIDE_PAR_X          64
+#define PAL_WIDE_PAR_Y          45
 
-#define NTSC_NORMAL_PAR_X       10
-#define NTSC_NORMAL_PAR_Y       11
-#define NTSC_WIDE_PAR_X         40
-#define NTSC_WIDE_PAR_Y         33
+#define NTSC_NORMAL_PAR_X       8
+#define NTSC_NORMAL_PAR_Y       9
+#define NTSC_WIDE_PAR_X         32
+#define NTSC_WIDE_PAR_Y         27
 
 #define DV_DEFAULT_QUALITY DV_QUALITY_BEST
 #define DV_DEFAULT_DECODE_NTH 1
@@ -131,6 +132,8 @@ gst_dvdec_quality_get_type (void)
 
 #define gst_dvdec_parent_class parent_class
 G_DEFINE_TYPE (GstDVDec, gst_dvdec, GST_TYPE_ELEMENT);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (dvdec, "dvdec", GST_RANK_MARGINAL,
+    GST_TYPE_DVDEC, dv_element_init (plugin));
 
 static GstFlowReturn gst_dvdec_chain (GstPad * pad, GstObject * parent,
     GstBuffer * buffer);
@@ -174,10 +177,8 @@ gst_dvdec_class_init (GstDVDecClass * klass)
 
   gstelement_class->change_state = GST_DEBUG_FUNCPTR (gst_dvdec_change_state);
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&sink_temp));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&src_temp));
+  gst_element_class_add_static_pad_template (gstelement_class, &sink_temp);
+  gst_element_class_add_static_pad_template (gstelement_class, &src_temp);
 
   gst_element_class_set_static_metadata (gstelement_class, "DV video decoder",
       "Codec/Decoder/Video",
@@ -185,6 +186,8 @@ gst_dvdec_class_init (GstDVDecClass * klass)
       "Erik Walthinsen <omega@cse.ogi.edu>," "Wim Taymans <wim@fluendo.com>");
 
   GST_DEBUG_CATEGORY_INIT (dvdec_debug, "dvdec", 0, "DV decoding element");
+
+  gst_type_mark_as_plugin_api (GST_TYPE_DVDEC_QUALITY, 0);
 }
 
 static void
@@ -305,6 +308,7 @@ static gboolean
 gst_dvdec_src_negotiate (GstDVDec * dvdec)
 {
   GstCaps *othercaps;
+  gboolean ret;
 
   /* no PAR was specified in input, derive from encoded data */
   if (dvdec->need_par) {
@@ -345,14 +349,14 @@ gst_dvdec_src_negotiate (GstDVDec * dvdec)
   }
 
   othercaps = gst_video_info_to_caps (&dvdec->vinfo);
-  gst_pad_set_caps (dvdec->srcpad, othercaps);
+  ret = gst_pad_set_caps (dvdec->srcpad, othercaps);
 
   gst_dvdec_negotiate_pool (dvdec, othercaps, &dvdec->vinfo);
   gst_caps_unref (othercaps);
 
   dvdec->src_negotiated = TRUE;
 
-  return TRUE;
+  return ret;
 }
 
 static gboolean
@@ -480,7 +484,7 @@ gst_dvdec_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
 
     caps = gst_pad_get_current_caps (dvdec->srcpad);
     if (!caps)
-      goto not_negotiated;
+      goto flushing;
 
     gst_dvdec_negotiate_pool (dvdec, caps, &dvdec->vinfo);
     gst_caps_unref (caps);
@@ -558,7 +562,16 @@ parse_header_error:
 not_negotiated:
   {
     GST_DEBUG_OBJECT (dvdec, "could not negotiate output");
-    ret = GST_FLOW_NOT_NEGOTIATED;
+    if (GST_PAD_IS_FLUSHING (dvdec->srcpad))
+      ret = GST_FLOW_FLUSHING;
+    else
+      ret = GST_FLOW_NOT_NEGOTIATED;
+    goto done;
+  }
+flushing:
+  {
+    GST_DEBUG_OBJECT (dvdec, "have no current caps");
+    ret = GST_FLOW_FLUSHING;
     goto done;
   }
 no_buffer:
