@@ -28,6 +28,7 @@
 #endif
 
 #include <gst/gl/gstglfuncs.h>
+#include <gst/video/gstvideoaffinetransformationmeta.h>
 
 #include "gstgtkglsink.h"
 #include "gtkgstglwidget.h"
@@ -35,6 +36,11 @@
 GST_DEBUG_CATEGORY (gst_debug_gtk_gl_sink);
 #define GST_CAT_DEFAULT gst_debug_gtk_gl_sink
 
+static void gst_gtk_gl_sink_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * pspec);
+static void gst_gtk_gl_sink_get_property (GObject * object, guint prop_id,
+    GValue * value, GParamSpec * pspec);
+
 static gboolean gst_gtk_gl_sink_start (GstBaseSink * bsink);
 static gboolean gst_gtk_gl_sink_stop (GstBaseSink * bsink);
 static gboolean gst_gtk_gl_sink_query (GstBaseSink * bsink, GstQuery * query);
@@ -42,6 +48,9 @@ static gboolean gst_gtk_gl_sink_propose_allocation (GstBaseSink * bsink,
     GstQuery * query);
 static GstCaps *gst_gtk_gl_sink_get_caps (GstBaseSink * bsink,
     GstCaps * filter);
+static gboolean gst_gtk_gl_sink_event (GstBaseSink * sink, GstEvent * event);
+
+static void gst_gtk_gl_sink_finalize (GObject * object);
 
 static GstStaticPadTemplate gst_gtk_gl_sink_template =
     GST_STATIC_PAD_TEMPLATE ("sink",
@@ -57,27 +66,57 @@ static GstStaticPadTemplate gst_gtk_gl_sink_template =
 G_DEFINE_TYPE_WITH_CODE (GstGtkGLSink, gst_gtk_gl_sink,
     GST_TYPE_GTK_BASE_SINK, GST_DEBUG_CATEGORY_INIT (gst_debug_gtk_gl_sink,
         "gtkglsink", 0, "Gtk GL Video Sink"));
+GST_ELEMENT_REGISTER_DEFINE (gtkglsink, "gtkglsink", GST_RANK_NONE,
+    GST_TYPE_GTK_GL_SINK);
+
+
+enum
+{
+  PROP_0,
+  PROP_ROTATE_METHOD,
+};
 
 static void
 gst_gtk_gl_sink_class_init (GstGtkGLSinkClass * klass)
 {
+  GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
   GstBaseSinkClass *gstbasesink_class;
   GstGtkBaseSinkClass *gstgtkbasesink_class;
 
+  gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
   gstbasesink_class = (GstBaseSinkClass *) klass;
   gstgtkbasesink_class = (GstGtkBaseSinkClass *) klass;
 
+  gobject_class->set_property = gst_gtk_gl_sink_set_property;
+  gobject_class->get_property = gst_gtk_gl_sink_get_property;
+  gobject_class->finalize = gst_gtk_gl_sink_finalize;
+
   gstbasesink_class->query = gst_gtk_gl_sink_query;
   gstbasesink_class->propose_allocation = gst_gtk_gl_sink_propose_allocation;
   gstbasesink_class->start = gst_gtk_gl_sink_start;
   gstbasesink_class->stop = gst_gtk_gl_sink_stop;
   gstbasesink_class->get_caps = gst_gtk_gl_sink_get_caps;
+  gstbasesink_class->event = gst_gtk_gl_sink_event;
 
   gstgtkbasesink_class->create_widget = gtk_gst_gl_widget_new;
   gstgtkbasesink_class->window_title = "Gtk+ GL renderer";
 
+  /**
+   * gtkglsink:rotate-method:
+   *
+   * Rotation method #GstVideoOrientationMethod used to render the media
+   *
+   * Since: 1.20
+   */
+  g_object_class_install_property (gobject_class, PROP_ROTATE_METHOD,
+      g_param_spec_enum ("rotate-method",
+          "rotate method",
+          "rotate method",
+          GST_TYPE_VIDEO_ORIENTATION_METHOD, GST_VIDEO_ORIENTATION_IDENTITY,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   gst_element_class_set_metadata (gstelement_class, "Gtk GL Video Sink",
       "Sink/Video", "A video sink that renders to a GtkWidget using OpenGL",
       "Matthew Waters <matthew@centricular.com>");
@@ -91,6 +130,55 @@ gst_gtk_gl_sink_init (GstGtkGLSink * gtk_sink)
 {
 }
 
+static void
+gst_gtk_gl_sink_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * pspec)
+{
+  switch (prop_id) {
+    case PROP_ROTATE_METHOD:
+    {
+      GtkWidget *widget =
+          gst_gtk_base_sink_acquire_widget (GST_GTK_BASE_SINK (object));
+      if (widget != NULL) {
+        gtk_gst_gl_widget_set_rotate_method (GTK_GST_GL_WIDGET (widget),
+            g_value_get_enum (value), FALSE);
+        g_object_unref (widget);
+      }
+      break;
+    }
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+}
+
+
+static void
+gst_gtk_gl_sink_get_property (GObject * object, guint prop_id,
+    GValue * value, GParamSpec * pspec)
+{
+  switch (prop_id) {
+    case PROP_ROTATE_METHOD:
+    {
+      GtkWidget *widget =
+          gst_gtk_base_sink_acquire_widget (GST_GTK_BASE_SINK (object));
+
+      if (widget != NULL) {
+        g_value_set_enum (value,
+            gtk_gst_gl_widget_get_rotate_method (GTK_GST_GL_WIDGET (widget)));
+        g_object_unref (widget);
+      } else {
+        g_value_set_enum (value, GST_VIDEO_ORIENTATION_IDENTITY);
+      }
+
+      break;
+    }
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+}
+
 static gboolean
 gst_gtk_gl_sink_query (GstBaseSink * bsink, GstQuery * query)
 {
@@ -138,6 +226,20 @@ _size_changed_cb (GtkWidget * widget, GdkRectangle * rectangle,
   }
 }
 
+static void
+destroy_cb (GtkWidget * widget, GstGtkGLSink * gtk_sink)
+{
+  if (gtk_sink->size_allocate_sig_handler) {
+    g_signal_handler_disconnect (widget, gtk_sink->size_allocate_sig_handler);
+    gtk_sink->size_allocate_sig_handler = 0;
+  }
+
+  if (gtk_sink->widget_destroy_sig_handler) {
+    g_signal_handler_disconnect (widget, gtk_sink->widget_destroy_sig_handler);
+    gtk_sink->widget_destroy_sig_handler = 0;
+  }
+}
+
 static gboolean
 gst_gtk_gl_sink_start (GstBaseSink * bsink)
 {
@@ -152,8 +254,14 @@ gst_gtk_gl_sink_start (GstBaseSink * bsink)
   gst_widget = GTK_GST_GL_WIDGET (base_sink->widget);
 
   /* Track the allocation size */
-  g_signal_connect (gst_widget, "size-allocate", G_CALLBACK (_size_changed_cb),
+  gtk_sink->size_allocate_sig_handler =
+      g_signal_connect (gst_widget, "size-allocate",
+      G_CALLBACK (_size_changed_cb), gtk_sink);
+
+  gtk_sink->widget_destroy_sig_handler =
+      g_signal_connect (gst_widget, "destroy", G_CALLBACK (destroy_cb),
       gtk_sink);
+
   _size_changed_cb (GTK_WIDGET (gst_widget), NULL, gtk_sink);
 
   if (!gtk_gst_gl_widget_init_winsys (gst_widget)) {
@@ -267,6 +375,8 @@ gst_gtk_gl_sink_propose_allocation (GstBaseSink * bsink, GstQuery * query)
 
   /* we also support various metadata */
   gst_query_add_allocation_meta (query, GST_VIDEO_META_API_TYPE, 0);
+  gst_query_add_allocation_meta (query,
+      GST_VIDEO_AFFINE_TRANSFORMATION_META_API_TYPE, 0);
 
   if (gtk_sink->context->gl_vtable->FenceSync)
     gst_query_add_allocation_meta (query, GST_GL_SYNC_META_API_TYPE, 0);
@@ -315,3 +425,62 @@ gst_gtk_gl_sink_get_caps (GstBaseSink * bsink, GstCaps * filter)
 
   return result;
 }
+
+static void
+gst_gtk_gl_sink_finalize (GObject * object)
+{
+  GstGtkGLSink *gtk_sink = GST_GTK_GL_SINK (object);
+  GstGtkBaseSink *base_sink = GST_GTK_BASE_SINK (object);
+
+  if (gtk_sink->size_allocate_sig_handler) {
+    g_signal_handler_disconnect (base_sink->widget,
+        gtk_sink->size_allocate_sig_handler);
+    gtk_sink->size_allocate_sig_handler = 0;
+  }
+
+  if (gtk_sink->widget_destroy_sig_handler) {
+    g_signal_handler_disconnect (base_sink->widget,
+        gtk_sink->widget_destroy_sig_handler);
+    gtk_sink->widget_destroy_sig_handler = 0;
+  }
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
+
+static gboolean
+gst_gtk_gl_sink_event (GstBaseSink * sink, GstEvent * event)
+{
+  GstTagList *taglist;
+  GstVideoOrientationMethod orientation;
+  gboolean ret;
+  GtkGstGLWidget *widget;
+
+  GST_DEBUG_OBJECT (sink, "handling %s event", GST_EVENT_TYPE_NAME (event));
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_TAG:
+      gst_event_parse_tag (event, &taglist);
+
+      if (gst_video_orientation_from_tag (taglist, &orientation)) {
+
+        widget = GTK_GST_GL_WIDGET
+            (gst_gtk_base_sink_acquire_widget (GST_GTK_BASE_SINK (sink)));
+        if (widget == NULL) {
+          GST_ERROR_OBJECT (sink, "Could not ensure GTK initialization.");
+          break;
+        }
+
+        gtk_gst_gl_widget_set_rotate_method (widget, orientation, TRUE);
+
+        g_object_unref (widget);
+      }
+      break;
+    default:
+      break;
+  }
+
+  ret = GST_BASE_SINK_CLASS (parent_class)->event (sink, event);
+
+  return ret;
+}
