@@ -11,8 +11,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 
@@ -29,15 +29,12 @@
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  if (!gst_element_register (plugin, "pngdec", GST_RANK_PRIMARY,
-          GST_TYPE_PNGDEC))
-    return FALSE;
+  gboolean ret = FALSE;
 
-  if (!gst_element_register (plugin, "pngenc", GST_RANK_PRIMARY,
-          GST_TYPE_PNGENC))
-    return FALSE;
+  ret |= GST_ELEMENT_REGISTER (pngdec, plugin);
+  ret |= GST_ELEMENT_REGISTER (pngenc, plugin);
 
-  return TRUE;
+  return ret;
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
