@@ -103,9 +103,10 @@ typedef enum _AtomsTreeFlavor
 typedef struct _AtomsContext
 {
   AtomsTreeFlavor flavor;
+  gboolean force_create_timecode_trak;
 } AtomsContext;
 
-AtomsContext* atoms_context_new  (AtomsTreeFlavor flavor);
+AtomsContext* atoms_context_new  (AtomsTreeFlavor flavor, gboolean force_create_timecode_trak);
 void          atoms_context_free (AtomsContext *context);
 
 #define METADATA_DATA_FLAG 0x0
@@ -318,11 +319,19 @@ typedef struct _AtomGMHD
 {
   Atom header;
 
+  /* Only gmin is required in a gmhd atom
+   * The other fields are optional */
   AtomGMIN gmin;
-  AtomTMCD tmcd;
+  AtomTMCD *tmcd;
 
 } AtomGMHD;
 
+typedef struct _AtomNMHD
+{
+  Atom header;
+  guint32 flags;
+} AtomNMHD;
+
 typedef struct _AtomURL
 {
   AtomFull header;
@@ -384,7 +393,8 @@ typedef enum _SampleEntryKind
   AUDIO,
   VIDEO,
   SUBTITLE,
-  TIMECODE
+  TIMECODE,
+  CLOSEDCAPTION
 } SampleEntryKind;
 
 typedef struct _SampleTableEntry
@@ -538,13 +548,17 @@ typedef struct _AtomTREF
 
 /*
  * used for both STCO and CO64
- * if used as STCO, entries should be truncated to use only 32bits
+ * The table will be written out as STCO automatically when
+ * the offsets being written will fit in a 32-bit table,
+ * otherwise it is written as CO64
  */
 typedef struct _AtomSTCO64
 {
   AtomFull header;
   /* Global offset to add to entries when serialising */
   guint32 chunk_offset;
+  /* Maximum offset stored in the table */
+  guint64 max_offset;
   ATOM_ARRAY (guint64) entries;
 } AtomSTCO64;
 
@@ -563,6 +577,14 @@ typedef struct _AtomCTTS
   gboolean do_pts;
 } AtomCTTS;
 
+typedef struct _AtomSVMI
+{
+  AtomFull header;
+
+  guint8 stereoscopic_composition_type;
+  gboolean is_left_first;
+} AtomSVMI;
+
 typedef struct _AtomSTBL
 {
   Atom header;
@@ -574,6 +596,8 @@ typedef struct _AtomSTBL
   AtomSTSZ stsz;
   /* NULL if not present */
   AtomCTTS *ctts;
+  /* NULL if not present */
+  AtomSVMI *svmi;
 
   AtomSTCO64 stco64;
 } AtomSTBL;
@@ -587,6 +611,7 @@ typedef struct _AtomMINF
   AtomSMHD *smhd;
   AtomHMHD *hmhd;
   AtomGMHD *gmhd;
+  AtomNMHD *nmhd;
 
   AtomHDLR *hdlr;
   AtomDINF dinf;
@@ -810,7 +835,7 @@ typedef struct _AtomTRAF
 
   AtomTFDT tfdt;
 
-  /* list of AtomTRUN */
+  /* list of AtomTRUN. */
   GList *truns;
   /* list of AtomSDTP */
   GList *sdtps;
@@ -824,6 +849,8 @@ typedef struct _AtomMOOF
 
   /* list of AtomTRAF */
   GList *trafs;
+
+  guint64 traf_offset;
 } AtomMOOF;
 
 
@@ -903,6 +930,8 @@ struct _AtomInfo
   AtomFreeFunc free_func;
 };
 
+guint64    atoms_get_current_qt_time   (void);
+
 guint64    atom_copy_data              (Atom *atom, guint8 **buffer,
                                         guint64 *size, guint64* offset);
 
@@ -921,10 +950,13 @@ void       atom_trak_set_elst_entry    (AtomTRAK * trak, gint index, guint32 dur
 void       atom_trak_edts_clear        (AtomTRAK * trak);
 guint32    atom_trak_get_timescale     (AtomTRAK *trak);
 guint32    atom_trak_get_id            (AtomTRAK * trak);
+void       atom_trak_set_constant_size_samples (AtomTRAK * trak, guint32 sample_size);
 void       atom_stbl_add_samples       (AtomSTBL * stbl, guint32 nsamples,
                                         guint32 delta, guint32 size,
                                         guint64 chunk_offset, gboolean sync,
                                         gint64 pts_offset);
+void       atom_stsc_add_new_entry     (AtomSTSC * stsc,
+                                        guint32 first_chunk, guint32 nsamples, guint32 sample_description_index);
 
 AtomMOOV*  atom_moov_new               (AtomsContext *context);
 void       atom_moov_free              (AtomMOOV *moov);
@@ -936,6 +968,8 @@ void       atom_moov_chunks_set_offset (AtomMOOV *moov, guint32 offset);
 void       atom_moov_add_trak          (AtomMOOV *moov, AtomTRAK *trak);
 guint      atom_moov_get_trak_count    (AtomMOOV *moov);
 
+guint      atom_framerate_to_timescale (gint fps_n, gint fps_d);
+
 guint64    atom_mvhd_copy_data         (AtomMVHD * atom, guint8 ** buffer,
                                         guint64 * size, guint64 * offset);
 void       atom_stco64_chunks_set_offset (AtomSTCO64 * stco64, guint32 offset);
@@ -953,18 +987,23 @@ guint64    atom_stsz_copy_data         (AtomSTSZ *atom, guint8 **buffer,
                                         guint64 *size, guint64* offset);
 guint64    atom_ctts_copy_data         (AtomCTTS *atom, guint8 **buffer,
                                         guint64 *size, guint64* offset);
+guint64    atom_svmi_copy_data         (AtomSVMI *atom, guint8 **buffer,
+                                        guint64 *size, guint64* offset);
+AtomSVMI * atom_svmi_new (guint8 stereoscopic_composition_type, gboolean is_left_first);
 guint64    atom_stco64_copy_data       (AtomSTCO64 *atom, guint8 **buffer,
                                         guint64 *size, guint64* offset);
 AtomMOOF*  atom_moof_new               (AtomsContext *context, guint32 sequence_number);
 void       atom_moof_free              (AtomMOOF *moof);
 guint64    atom_moof_copy_data         (AtomMOOF *moof, guint8 **buffer, guint64 *size, guint64* offset);
+void       atom_moof_set_base_offset   (AtomMOOF * moof, guint64 offset);
 AtomTRAF * atom_traf_new               (AtomsContext * context, guint32 track_ID);
 void       atom_traf_free              (AtomTRAF * traf);
 void       atom_traf_set_base_decode_time (AtomTRAF * traf, guint64 base_decode_time);
-void       atom_traf_add_samples       (AtomTRAF * traf, guint32 delta,
-                                        guint32 size, gboolean sync, gint64 pts_offset,
-                                        gboolean sdtp_sync);
+void       atom_traf_add_samples       (AtomTRAF * traf, guint32 nsamples, guint32 delta,
+                                        guint32 size, gint32 data_offset, gboolean sync,
+                                        gint64 pts_offset, gboolean sdtp_sync);
 guint32    atom_traf_get_sample_num    (AtomTRAF * traf);
+void       atom_trun_set_offset        (AtomTRUN * trun, gint32 offset);
 void       atom_moof_add_traf          (AtomMOOF *moof, AtomTRAF *traf);
 
 AtomMFRA*  atom_mfra_new               (AtomsContext *context);
@@ -977,7 +1016,6 @@ guint64    atom_mfra_copy_data         (AtomMFRA *mfra, guint8 **buffer, guint64
 
 
 /* media sample description related helpers */
-
 typedef struct
 {
   guint16 version;
@@ -1032,7 +1070,10 @@ SampleTableEntryTX3G * atom_trak_set_subtitle_type (AtomTRAK * trak, AtomsContex
                                SubtitleSampleEntry * entry);
 
 SampleTableEntryTMCD *
-atom_trak_set_timecode_type (AtomTRAK * trak, AtomsContext * context, GstVideoTimeCode * tc);
+atom_trak_set_timecode_type (AtomTRAK * trak, AtomsContext * context, guint trak_timescale, GstVideoTimeCode * tc);
+
+SampleTableEntry * atom_trak_set_caption_type (AtomTRAK *trak, AtomsContext *context,
+					       guint32 trak_timescale, guint32 caption_type);
 
 void atom_trak_update_bitrates (AtomTRAK * trak, guint32 avg_bitrate,
                                 guint32 max_bitrate);
@@ -1077,6 +1118,10 @@ AtomInfo *   build_SMI_atom              (const GstBuffer *seqh);
 AtomInfo *   build_ima_adpcm_extension   (gint channels, gint rate,
                                           gint blocksize);
 AtomInfo *   build_uuid_xmp_atom         (GstBuffer * xmp);
+AtomInfo *   build_vpcC_extension        (guint8 profile, guint8 level, guint8 bit_depth,
+                                          guint8 chroma_subsampling, gboolean video_full_range,
+                                          guint8 colour_primaries, guint8 transfer_characteristics,
+                                          guint8 matrix_coefficients);
 
 
 /*
