@@ -37,13 +37,8 @@ GST_DEBUG_CATEGORY_STATIC (splitmux_part_debug);
 #define SPLITMUX_PART_TYPE_LOCK(p) g_mutex_lock(&(p)->type_lock)
 #define SPLITMUX_PART_TYPE_UNLOCK(p) g_mutex_unlock(&(p)->type_lock)
 
-enum
-{
-  SIGNAL_PREPARED,
-  LAST_SIGNAL
-};
-
-static guint part_reader_signals[LAST_SIGNAL] = { 0 };
+#define SPLITMUX_PART_MSG_LOCK(p) g_mutex_lock(&(p)->msg_lock)
+#define SPLITMUX_PART_MSG_UNLOCK(p) g_mutex_unlock(&(p)->msg_lock)
 
 typedef struct _GstSplitMuxPartPad
 {
@@ -59,6 +54,7 @@ typedef struct _GstSplitMuxPartPad
   gboolean is_eos;
   gboolean flushing;
   gboolean seen_buffer;
+  gboolean first_activation;
 
   gboolean is_sparse;
   GstClockTime max_ts;
@@ -88,6 +84,10 @@ static void type_found (GstElement * typefind, guint probability,
     GstCaps * caps, GstSplitMuxPartReader * reader);
 static void check_if_pads_collected (GstSplitMuxPartReader * reader);
 
+static void
+gst_splitmux_part_reader_finish_measuring_streams (GstSplitMuxPartReader *
+    reader);
+
 /* Called with reader lock held */
 static gboolean
 have_empty_queue (GstSplitMuxPartReader * reader)
@@ -140,7 +140,7 @@ handle_buffer_measuring (GstSplitMuxPartReader * reader,
   if (reader->prep_state == PART_STATE_PREPARING_COLLECT_STREAMS &&
       !part_pad->seen_buffer) {
     /* If this is the first buffer on the pad in the collect_streams state,
-     * then calculate inital offset based on running time of this segment */
+     * then calculate initial offset based on running time of this segment */
     part_pad->initial_ts_offset =
         part_pad->orig_segment.start + part_pad->orig_segment.base -
         part_pad->orig_segment.time;
@@ -153,18 +153,21 @@ handle_buffer_measuring (GstSplitMuxPartReader * reader,
   /* Adjust buffer timestamps */
   offset = reader->start_offset + part_pad->segment.base;
   offset -= part_pad->initial_ts_offset;
+  /* We don't add the ts_offset here, because we
+   * want to measure the logical length of the stream,
+   * not to generate output timestamps */
 
   /* Update the stored max duration on the pad,
-   * always preferring making DTS contiguous
+   * always preferring making reordered PTS contiguous
    * where possible */
-  if (GST_BUFFER_DTS_IS_VALID (buf))
-    ts = GST_BUFFER_DTS (buf) + offset;
-  else if (GST_BUFFER_PTS_IS_VALID (buf))
+  if (GST_BUFFER_PTS_IS_VALID (buf))
     ts = GST_BUFFER_PTS (buf) + offset;
+  else if (GST_BUFFER_DTS_IS_VALID (buf))
+    ts = GST_BUFFER_DTS (buf) + offset;
 
   GST_DEBUG_OBJECT (reader, "Pad %" GST_PTR_FORMAT
-      " incoming PTS %" GST_TIME_FORMAT
-      " DTS %" GST_TIME_FORMAT " offset by %" GST_STIME_FORMAT
+      " incoming DTS %" GST_TIME_FORMAT
+      " PTS %" GST_TIME_FORMAT " offset by %" GST_STIME_FORMAT
       " to %" GST_STIME_FORMAT, part_pad,
       GST_TIME_ARGS (GST_BUFFER_DTS (buf)),
       GST_TIME_ARGS (GST_BUFFER_PTS (buf)),
@@ -229,9 +232,16 @@ splitmux_part_pad_chain (GstPad * pad, GstObject * parent, GstBuffer * buf)
     return GST_FLOW_FLUSHING;
   }
 
+  if (GST_PAD_LAST_FLOW_RETURN (part_pad->target) == GST_FLOW_NOT_LINKED) {
+    SPLITMUX_PART_UNLOCK (reader);
+    gst_buffer_unref (buf);
+    return GST_FLOW_NOT_LINKED;
+  }
+
   /* Adjust buffer timestamps */
   offset = reader->start_offset + part_pad->segment.base;
   offset -= part_pad->initial_ts_offset;
+  offset += reader->ts_offset;
 
   if (GST_BUFFER_PTS_IS_VALID (buf))
     GST_BUFFER_PTS (buf) += offset;
@@ -271,7 +281,8 @@ splitmux_part_is_eos_locked (GstSplitMuxPartReader * part)
   GList *cur;
   for (cur = g_list_first (part->pads); cur != NULL; cur = g_list_next (cur)) {
     GstSplitMuxPartPad *part_pad = SPLITMUX_PART_PAD_CAST (cur->data);
-    if (!part_pad->is_eos)
+    if (GST_PAD_LAST_FLOW_RETURN (part_pad->target) != GST_FLOW_NOT_LINKED
+        && !part_pad->is_eos)
       return FALSE;
   }
 
@@ -322,14 +333,51 @@ splitmux_is_flushing (GstSplitMuxPartReader * reader)
   return FALSE;
 }
 
+static gboolean
+enqueue_event (GstSplitMuxPartReader * reader, GstSplitMuxPartPad * part_pad,
+    GstEvent * event)
+{
+  gboolean ret = TRUE;
+  GstDataQueueItem *item;
+
+  GST_LOG_OBJECT (reader, "Enqueueing event %" GST_PTR_FORMAT, event);
+  item = g_slice_new (GstDataQueueItem);
+  item->destroy = (GDestroyNotify) splitmux_part_free_queue_item;
+  item->object = GST_MINI_OBJECT (event);
+  item->size = 0;
+  item->duration = 0;
+  if (item->duration == GST_CLOCK_TIME_NONE)
+    item->duration = 0;
+  item->visible = FALSE;
+
+  if (!gst_data_queue_push (part_pad->queue, item)) {
+    splitmux_part_free_queue_item (item);
+    ret = FALSE;
+  }
+
+  return ret;
+}
+
+static gboolean
+resend_sticky_event (GstPad * pad, GstEvent ** event, gpointer user_data)
+{
+  GstSplitMuxPartPad *part_pad = SPLITMUX_PART_PAD_CAST (pad);
+  GstSplitMuxPartReader *reader = part_pad->reader;
+
+  GST_DEBUG_OBJECT (part_pad, "queuing sticky event %" GST_PTR_FORMAT, *event);
+  (void) enqueue_event (reader, part_pad, gst_event_ref (*event));
+
+  return TRUE;
+}
+
 static gboolean
 splitmux_part_pad_event (GstPad * pad, GstObject * parent, GstEvent * event)
 {
   GstSplitMuxPartPad *part_pad = SPLITMUX_PART_PAD_CAST (pad);
   GstSplitMuxPartReader *reader = part_pad->reader;
   gboolean ret = TRUE;
+  gboolean resend_sticky = FALSE;
   SplitMuxSrcPad *target;
-  GstDataQueueItem *item;
 
   SPLITMUX_PART_LOCK (reader);
 
@@ -360,21 +408,22 @@ splitmux_part_pad_event (GstPad * pad, GstObject * parent, GstEvent * event)
         goto wrong_segment;
 
       /* Adjust segment */
-      /* Adjust start/stop so the overall file is 0 + start_offset based */
+      /* Adjust start/stop so the overall file is 0 + start_offset based,
+       * adding a fixed offset so that DTS is never negative */
       if (seg->stop != -1) {
         seg->stop -= seg->start;
-        seg->stop += seg->time + reader->start_offset;
+        seg->stop += seg->time + reader->start_offset + reader->ts_offset;
       }
-      seg->start = seg->time + reader->start_offset;
+      seg->start = seg->time + reader->start_offset + reader->ts_offset;
       seg->time += reader->start_offset;
       seg->position += reader->start_offset;
 
-      GST_LOG_OBJECT (pad, "Adjusted segment now %" GST_PTR_FORMAT, event);
-
       /* Replace event */
       gst_event_unref (event);
       event = gst_event_new_segment (seg);
 
+      GST_LOG_OBJECT (pad, "Adjusted segment now %" GST_PTR_FORMAT, event);
+
       if (reader->prep_state != PART_STATE_PREPARING_COLLECT_STREAMS
           && reader->prep_state != PART_STATE_PREPARING_MEASURE_STREAMS)
         break;                  /* Only do further stuff with segments during initial measuring */
@@ -409,13 +458,18 @@ splitmux_part_pad_event (GstPad * pad, GstObject * parent, GstEvent * event)
           reader->prep_state == PART_STATE_PREPARING_MEASURE_STREAMS) {
         /* Mark this pad as EOS */
         part_pad->is_eos = TRUE;
+        /* Mark the pad to re-send sticky events on the first activation */
+        part_pad->first_activation = TRUE;
         if (splitmux_part_is_eos_locked (reader)) {
           /* Finished measuring things, set state and tell the state change func
            * so it can seek back to the start */
           GST_LOG_OBJECT (reader,
               "EOS while measuring streams. Resetting for ready");
           reader->prep_state = PART_STATE_PREPARING_RESET_FOR_READY;
-          SPLITMUX_PART_BROADCAST (reader);
+
+          gst_element_call_async (GST_ELEMENT_CAST (reader),
+              (GstElementCallAsyncFunc)
+              gst_splitmux_part_reader_finish_measuring_streams, NULL, NULL);
         }
         goto drop_event;
       }
@@ -456,12 +510,8 @@ splitmux_part_pad_event (GstPad * pad, GstObject * parent, GstEvent * event)
       GST_EVENT_TYPE (event) == GST_EVENT_FLUSH_STOP)
     goto drop_event;
 
-  if (!block_until_can_push (reader)) {
-    SPLITMUX_PART_UNLOCK (reader);
-    gst_object_unref (target);
-    gst_event_unref (event);
-    return FALSE;
-  }
+  if (!block_until_can_push (reader))
+    goto drop_event;
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_GAP:{
@@ -472,25 +522,22 @@ splitmux_part_pad_event (GstPad * pad, GstObject * parent, GstEvent * event)
       break;
   }
 
+  /* Re-queue any sticky events on the first activation that were
+   * dropped during probing */
+  if (part_pad->first_activation) {
+    resend_sticky = TRUE;
+    part_pad->first_activation = FALSE;
+  }
+
   /* We are active, and one queue is empty, place this buffer in
    * the dataqueue */
   gst_object_ref (part_pad->queue);
   SPLITMUX_PART_UNLOCK (reader);
 
-  GST_LOG_OBJECT (reader, "Enqueueing event %" GST_PTR_FORMAT, event);
-  item = g_slice_new (GstDataQueueItem);
-  item->destroy = (GDestroyNotify) splitmux_part_free_queue_item;
-  item->object = GST_MINI_OBJECT (event);
-  item->size = 0;
-  item->duration = 0;
-  if (item->duration == GST_CLOCK_TIME_NONE)
-    item->duration = 0;
-  item->visible = FALSE;
+  if (resend_sticky)
+    gst_pad_sticky_events_foreach (pad, resend_sticky_event, NULL);
 
-  if (!gst_data_queue_push (part_pad->queue, item)) {
-    splitmux_part_free_queue_item (item);
-    ret = FALSE;
-  }
+  ret = enqueue_event (reader, part_pad, event);
 
   gst_object_unref (part_pad->queue);
   gst_object_unref (target);
@@ -505,11 +552,11 @@ wrong_segment:
           reader->path, pad));
   return FALSE;
 drop_event:
+  SPLITMUX_PART_UNLOCK (reader);
   GST_LOG_OBJECT (pad, "Dropping event %" GST_PTR_FORMAT
       " from %" GST_PTR_FORMAT " on %" GST_PTR_FORMAT, event, pad, target);
   gst_event_unref (event);
   gst_object_unref (target);
-  SPLITMUX_PART_UNLOCK (reader);
   return TRUE;
 }
 
@@ -619,10 +666,6 @@ gst_splitmux_part_reader_class_init (GstSplitMuxPartReaderClass * klass)
   gobject_klass->dispose = splitmux_part_reader_dispose;
   gobject_klass->finalize = splitmux_part_reader_finalize;
 
-  part_reader_signals[SIGNAL_PREPARED] =
-      g_signal_new ("prepared", G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_FIRST, G_STRUCT_OFFSET (GstSplitMuxPartReaderClass,
-          prepared), NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
   gstelement_class->change_state = gst_splitmux_part_reader_change_state;
   gstelement_class->send_event = gst_splitmux_part_reader_send_event;
 
@@ -640,6 +683,7 @@ gst_splitmux_part_reader_init (GstSplitMuxPartReader * reader)
   g_cond_init (&reader->inactive_cond);
   g_mutex_init (&reader->lock);
   g_mutex_init (&reader->type_lock);
+  g_mutex_init (&reader->msg_lock);
 
   /* FIXME: Create elements on a state change */
   reader->src = gst_element_factory_make ("filesrc", NULL);
@@ -687,12 +731,44 @@ splitmux_part_reader_finalize (GObject * object)
   g_cond_clear (&reader->inactive_cond);
   g_mutex_clear (&reader->lock);
   g_mutex_clear (&reader->type_lock);
+  g_mutex_clear (&reader->msg_lock);
 
   g_free (reader->path);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static void
+do_async_start (GstSplitMuxPartReader * reader)
+{
+  GstMessage *message;
+
+  SPLITMUX_PART_MSG_LOCK (reader);
+  reader->async_pending = TRUE;
+
+  message = gst_message_new_async_start (GST_OBJECT_CAST (reader));
+  GST_BIN_CLASS (parent_class)->handle_message (GST_BIN_CAST (reader), message);
+  SPLITMUX_PART_MSG_UNLOCK (reader);
+}
+
+static void
+do_async_done (GstSplitMuxPartReader * reader)
+{
+  GstMessage *message;
+
+  SPLITMUX_PART_MSG_LOCK (reader);
+  if (reader->async_pending) {
+    message =
+        gst_message_new_async_done (GST_OBJECT_CAST (reader),
+        GST_CLOCK_TIME_NONE);
+    GST_BIN_CLASS (parent_class)->handle_message (GST_BIN_CAST (reader),
+        message);
+
+    reader->async_pending = FALSE;
+  }
+  SPLITMUX_PART_MSG_UNLOCK (reader);
+}
+
 static void
 splitmux_part_reader_reset (GstSplitMuxPartReader * reader)
 {
@@ -854,6 +930,7 @@ gst_splitmux_part_reader_seek_to_segment (GstSplitMuxPartReader * reader,
 static void
 gst_splitmux_part_reader_measure_streams (GstSplitMuxPartReader * reader)
 {
+  SPLITMUX_PART_LOCK (reader);
   /* Trigger a flushing seek to near the end of the file and run each stream
    * to EOS in order to find the smallest end timestamp to start the next
    * file from
@@ -863,18 +940,23 @@ gst_splitmux_part_reader_measure_streams (GstSplitMuxPartReader * reader)
     GstClockTime seek_ts = reader->duration - (0.5 * GST_SECOND);
     gst_splitmux_part_reader_seek_to_time_locked (reader, seek_ts);
   }
+  SPLITMUX_PART_UNLOCK (reader);
+}
 
-  /* Wait for things to happen */
-  while (reader->prep_state == PART_STATE_PREPARING_MEASURE_STREAMS)
-    SPLITMUX_PART_WAIT (reader);
-
+static void
+gst_splitmux_part_reader_finish_measuring_streams (GstSplitMuxPartReader *
+    reader)
+{
+  SPLITMUX_PART_LOCK (reader);
   if (reader->prep_state == PART_STATE_PREPARING_RESET_FOR_READY) {
     /* Fire the prepared signal and go to READY state */
     GST_DEBUG_OBJECT (reader,
-        "Stream measuring complete. File %s is now ready. Firing prepared signal",
-        reader->path);
+        "Stream measuring complete. File %s is now ready", reader->path);
     reader->prep_state = PART_STATE_READY;
-    g_signal_emit (reader, part_reader_signals[SIGNAL_PREPARED], 0, NULL);
+    SPLITMUX_PART_UNLOCK (reader);
+    do_async_done (reader);
+  } else {
+    SPLITMUX_PART_UNLOCK (reader);
   }
 }
 
@@ -943,7 +1025,9 @@ check_if_pads_collected (GstSplitMuxPartReader * reader)
       GST_DEBUG_OBJECT (reader,
           "no more pads - file %s. Measuring stream length", reader->path);
       reader->prep_state = PART_STATE_PREPARING_MEASURE_STREAMS;
-      SPLITMUX_PART_BROADCAST (reader);
+      gst_element_call_async (GST_ELEMENT_CAST (reader),
+          (GstElementCallAsyncFunc) gst_splitmux_part_reader_measure_streams,
+          NULL, NULL);
     }
   }
 }
@@ -1045,16 +1129,16 @@ gst_splitmux_part_reader_change_state (GstElement * element,
       break;
     }
     case GST_STATE_CHANGE_READY_TO_PAUSED:{
-      /* Hold the splitmux type lock until after the
-       * parent state change function has finished
-       * changing the states of things, and type finding can continue */
       SPLITMUX_PART_LOCK (reader);
       g_object_set (reader->src, "location", reader->path, NULL);
       reader->prep_state = PART_STATE_PREPARING_COLLECT_STREAMS;
       gst_splitmux_part_reader_set_flushing_locked (reader, FALSE);
       reader->running = TRUE;
       SPLITMUX_PART_UNLOCK (reader);
-      SPLITMUX_PART_TYPE_LOCK (reader);
+
+      /* we go to PAUSED asynchronously once all streams have been collected
+       * and seeks to measure the stream lengths are done */
+      do_async_start (reader);
       break;
     }
     case GST_STATE_CHANGE_READY_TO_NULL:
@@ -1078,33 +1162,16 @@ gst_splitmux_part_reader_change_state (GstElement * element,
 
   ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
   if (ret == GST_STATE_CHANGE_FAILURE) {
-    if (transition == GST_STATE_CHANGE_READY_TO_PAUSED) {
-      /* Make sure to release the lock we took above */
-      SPLITMUX_PART_TYPE_UNLOCK (reader);
-    }
+    do_async_done (reader);
     goto beach;
   }
 
   switch (transition) {
     case GST_STATE_CHANGE_READY_TO_PAUSED:
-      /* Sleep and wait until all streams have been collected, then do the seeks
-       * to measure the stream lengths. This took the type lock above,
-       * but it's OK to release it now and let typefinding happen... */
-      SPLITMUX_PART_TYPE_UNLOCK (reader);
-
-      SPLITMUX_PART_LOCK (reader);
-
-      while (reader->prep_state == PART_STATE_PREPARING_COLLECT_STREAMS) {
-        GST_LOG_OBJECT (reader, "Waiting to collect all output streams");
-        SPLITMUX_PART_WAIT (reader);
-      }
-
-      if (reader->prep_state == PART_STATE_PREPARING_MEASURE_STREAMS ||
-          reader->prep_state == PART_STATE_PREPARING_RESET_FOR_READY) {
-        gst_splitmux_part_reader_measure_streams (reader);
-      } else if (reader->prep_state == PART_STATE_FAILED)
-        ret = GST_STATE_CHANGE_FAILURE;
-      SPLITMUX_PART_UNLOCK (reader);
+      ret = GST_STATE_CHANGE_ASYNC;
+      break;
+    case GST_STATE_CHANGE_PAUSED_TO_READY:
+      do_async_done (reader);
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
       SPLITMUX_PART_LOCK (reader);
@@ -1125,28 +1192,6 @@ beach:
   return ret;
 }
 
-static gboolean
-check_bus_messages (GstSplitMuxPartReader * part)
-{
-  gboolean ret = FALSE;
-  GstBus *bus;
-  GstMessage *m;
-
-  bus = gst_element_get_bus (GST_ELEMENT_CAST (part));
-  while ((m = gst_bus_pop (bus)) != NULL) {
-    if (GST_MESSAGE_TYPE (m) == GST_MESSAGE_ERROR) {
-      GST_LOG_OBJECT (part, "Got error message while preparing. Failing.");
-      gst_message_unref (m);
-      goto done;
-    }
-    gst_message_unref (m);
-  }
-  ret = TRUE;
-done:
-  gst_object_unref (bus);
-  return ret;
-}
-
 gboolean
 gst_splitmux_part_reader_prepare (GstSplitMuxPartReader * part)
 {
@@ -1154,10 +1199,10 @@ gst_splitmux_part_reader_prepare (GstSplitMuxPartReader * part)
 
   ret = gst_element_set_state (GST_ELEMENT_CAST (part), GST_STATE_PAUSED);
 
-  if (ret != GST_STATE_CHANGE_SUCCESS)
+  if (ret == GST_STATE_CHANGE_FAILURE)
     return FALSE;
 
-  return check_bus_messages (part);
+  return TRUE;
 }
 
 void
@@ -1255,12 +1300,13 @@ gst_splitmux_part_reader_get_end_offset (GstSplitMuxPartReader * reader)
 
 void
 gst_splitmux_part_reader_set_start_offset (GstSplitMuxPartReader * reader,
-    GstClockTime offset)
+    GstClockTime time_offset, GstClockTime ts_offset)
 {
   SPLITMUX_PART_LOCK (reader);
-  reader->start_offset = offset;
-  GST_INFO_OBJECT (reader, "TS offset now %" GST_TIME_FORMAT,
-      GST_TIME_ARGS (offset));
+  reader->start_offset = time_offset;
+  reader->ts_offset = ts_offset;
+  GST_INFO_OBJECT (reader, "Time offset now %" GST_TIME_FORMAT,
+      GST_TIME_ARGS (time_offset));
   SPLITMUX_PART_UNLOCK (reader);
 }
 
@@ -1360,9 +1406,12 @@ bus_handler (GstBin * bin, GstMessage * message)
       /* Make sure to set the state to failed and wake up the listener
        * on error */
       SPLITMUX_PART_LOCK (reader);
+      GST_ERROR_OBJECT (reader, "Got error message from child %" GST_PTR_FORMAT
+          " marking this reader as failed", GST_MESSAGE_SRC (message));
       reader->prep_state = PART_STATE_FAILED;
       SPLITMUX_PART_BROADCAST (reader);
       SPLITMUX_PART_UNLOCK (reader);
+      do_async_done (reader);
       break;
     default:
       break;
