@@ -22,17 +22,17 @@
 
 /**
  * SECTION:element-osssrc
+ * @title: osssrc
  *
  * This element lets you record sound using the Open Sound System (OSS).
  *
- * <refsect2>
- * <title>Example pipelines</title>
+ * ## Example pipelines
  * |[
  * gst-launch-1.0 -v osssrc ! audioconvert ! vorbisenc ! oggmux ! filesink location=mymusic.ogg
  * ]| will record sound from your sound card using OSS and encode it to an
  * Ogg/Vorbis file (this will only work if your mixer settings are right
  * and the right inputs enabled etc.)
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -60,9 +60,10 @@
 #endif /* HAVE_OSS_INCLUDE_IN_SYS */
 
 #include "common.h"
+#include "gstossaudioelements.h"
 #include "gstosssrc.h"
 
-#include <gst/gst-i18n-plugin.h>
+#include <glib/gi18n-lib.h>
 
 GST_DEBUG_CATEGORY_EXTERN (oss_debug);
 #define GST_CAT_DEFAULT oss_debug
@@ -79,6 +80,8 @@ enum
 
 #define gst_oss_src_parent_class parent_class
 G_DEFINE_TYPE (GstOssSrc, gst_oss_src, GST_TYPE_AUDIO_SRC);
+GST_ELEMENT_REGISTER_DEFINE_WITH_CODE (osssrc, "osssrc", GST_RANK_SECONDARY,
+    GST_TYPE_OSS_SRC, oss_element_init (plugin));
 
 static void gst_oss_src_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
@@ -167,8 +170,8 @@ gst_oss_src_class_init (GstOssSrcClass * klass)
       "Capture from a sound card via OSS",
       "Erik Walthinsen <omega@cse.ogi.edu>, " "Wim Taymans <wim@fluendo.com>");
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&osssrc_src_factory));
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &osssrc_src_factory);
 }
 
 static void
