@@ -2,7 +2,7 @@
  * OSX video sink
  * Copyright (C) 2004-6 Zaheer Abbas Merali <zaheerabbas at merali dot org>
  * Copyright (C) 2007,2008,2009 Pioneers of the Inevitable <songbird@songbirdnest.com>
- * 
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
  * License as published by the Free Software Foundation; either
@@ -20,7 +20,7 @@
  *
  * The development of this code was made possible due to the involvement of
  * Pioneers of the Inevitable, the creators of the Songbird Music player.
- * 
+ *
  */
 
 /**
@@ -29,16 +29,12 @@
  * The OSXVideoSink renders video frames to a MacOSX window. The video output
  * must be directed to a window embedded in an existing NSApp.
  *
- * When the NSView to be embedded is created an element #GstMessage with a 
- * name of 'have-ns-view' will be created and posted on the bus. 
- * The pointer to the NSView to embed will be in the 'nsview' field of that 
- * message. The application MUST handle this message and embed the view
- * appropriately.
  */
 
 #include "config.h"
 #include <gst/video/videooverlay.h>
 #include <gst/video/navigation.h>
+#include <gst/video/video.h>
 
 #include "osxvideosink.h"
 #include <unistd.h>
@@ -47,14 +43,12 @@
 GST_DEBUG_CATEGORY (gst_debug_osx_video_sink);
 #define GST_CAT_DEFAULT gst_debug_osx_video_sink
 
-#ifdef RUN_NS_APP_THREAD
+#ifndef GSTREAMER_GLIB_COCOA_NSAPPLICATION
 #include <pthread.h>
-extern  void _CFRunLoopSetCurrent(CFRunLoopRef rl);
+extern void _CFRunLoopSetCurrent (CFRunLoopRef rl);
 extern pthread_t _CFMainPThread;
 #endif
 
-
-
 static GstStaticPadTemplate gst_osx_video_sink_sink_template_factory =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -78,18 +72,30 @@ enum
 };
 
 static void gst_osx_video_sink_osxwindow_destroy (GstOSXVideoSink * osxvideosink);
+
+#ifndef GSTREAMER_GLIB_COCOA_NSAPPLICATION
 static GMutex _run_loop_check_mutex;
 static GMutex _run_loop_mutex;
 static GCond _run_loop_cond;
+#endif
+
 static GstOSXVideoSinkClass *sink_class = NULL;
 static GstVideoSinkClass *parent_class = NULL;
 
+#if MAC_OS_X_VERSION_MAX_ALLOWED < 101200
+#define NSEventMaskAny                       NSAnyEventMask
+#define NSWindowStyleMaskTitled              NSTitledWindowMask
+#define NSWindowStyleMaskClosable            NSClosableWindowMask
+#define NSWindowStyleMaskResizable           NSResizableWindowMask
+#define NSWindowStyleMaskTexturedBackground  NSTexturedBackgroundWindowMask
+#define NSWindowStyleMaskMiniaturizable      NSMiniaturizableWindowMask
+#endif
+
 /* Helper to trigger calls from the main thread */
 static void
 gst_osx_video_sink_call_from_main_thread(GstOSXVideoSink *osxvideosink,
     NSObject * object, SEL function, NSObject *data, BOOL waitUntilDone)
 {
-
   NSThread *thread;
   NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
 
@@ -104,6 +110,7 @@ gst_osx_video_sink_call_from_main_thread(GstOSXVideoSink *osxvideosink,
   [pool release];
 }
 
+#ifndef GSTREAMER_GLIB_COCOA_NSAPPLICATION
 /* Poll for cocoa events */
 static void
 run_ns_app_loop (void) {
@@ -111,15 +118,11 @@ run_ns_app_loop (void) {
   NSAutoreleasePool *pool =[[NSAutoreleasePool alloc] init];
   NSDate *pollTime = nil;
 
-#ifdef RUN_NS_APP_THREAD
   /* when running the loop in a thread we want to sleep as long as possible */
   pollTime = [NSDate distantFuture];
-#else
-  pollTime = [NSDate distantPast];
-#endif
 
   do {
-      event = [NSApp nextEventMatchingMask:NSAnyEventMask untilDate:pollTime
+      event = [NSApp nextEventMatchingMask:NSEventMaskAny untilDate:pollTime
           inMode:NSDefaultRunLoopMode dequeue:YES];
       [NSApp sendEvent:event];
     }
@@ -196,7 +199,6 @@ gst_osx_video_sink_run_cocoa_loop (GstOSXVideoSink * sink )
   }
 
   if (sink_class->ns_app_thread == NULL) {
-#ifdef RUN_NS_APP_THREAD
     /* run the main runloop in a separate thread */
 
     /* override [NSThread isMainThread] with our own implementation so that we can
@@ -217,12 +219,6 @@ gst_osx_video_sink_run_cocoa_loop (GstOSXVideoSink * sink )
     g_mutex_lock (&_run_loop_mutex);
     g_cond_wait (&_run_loop_cond, &_run_loop_mutex);
     g_mutex_unlock (&_run_loop_mutex);
-#else
-  /* assume that there is a GMainLoop and iterate the main runloop from there
-   */
-    sink_class->cocoa_timeout = g_timeout_add (10,
-        (GSourceFunc) run_ns_app_loop, NULL);
-#endif
   }
 
   g_mutex_unlock (&_run_loop_check_mutex);
@@ -231,11 +227,8 @@ gst_osx_video_sink_run_cocoa_loop (GstOSXVideoSink * sink )
 static void
 gst_osx_video_sink_stop_cocoa_loop (GstOSXVideoSink * osxvideosink)
 {
-#ifndef RUN_NS_APP_THREAD
-  if (sink_class->cocoa_timeout)
-    g_source_remove(sink_klass->cocoa_timeout);
-#endif
 }
+#endif
 
 /* This function handles osx window creation */
 static gboolean
@@ -244,8 +237,6 @@ gst_osx_video_sink_osxwindow_create (GstOSXVideoSink * osxvideosink, gint width,
 {
   NSRect rect;
   GstOSXWindow *osxwindow = NULL;
-  GstStructure *s;
-  GstMessage *msg;
   gboolean res = TRUE;
   NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
 
@@ -268,52 +259,35 @@ gst_osx_video_sink_osxwindow_create (GstOSXVideoSink * osxvideosink, gint width,
   rect.size.height = (float) osxwindow->height;
   osxwindow->gstview =[[GstGLView alloc] initWithFrame:rect];
 
-  s = gst_structure_new ("have-ns-view",
-     "nsview", G_TYPE_POINTER, osxwindow->gstview,
-     nil);
-
-  msg = gst_message_new_element (GST_OBJECT (osxvideosink), s);
-  gst_element_post_message (GST_ELEMENT (osxvideosink), msg);
-
-  GST_INFO_OBJECT (osxvideosink, "'have-ns-view' message sent");
-
+#ifndef GSTREAMER_GLIB_COCOA_NSAPPLICATION
   gst_osx_video_sink_run_cocoa_loop (osxvideosink);
   [osxwindow->gstview setMainThread:sink_class->ns_app_thread];
+#endif
 
-  /* check if have-ns-view was handled and osxwindow->gstview was added to a
-   * superview
-   */
-  if ([osxwindow->gstview haveSuperview] == NO) {
-    /* have-ns-view wasn't handled, post prepare-xwindow-id */
-    if (osxvideosink->superview == NULL) {
-      GST_INFO_OBJECT (osxvideosink, "emitting prepare-xwindow-id");
-      gst_video_overlay_prepare_window_handle (GST_VIDEO_OVERLAY (osxvideosink));
-    }
-
-    if (osxvideosink->superview != NULL) {
-      /* prepare-xwindow-id was handled, we have the superview in
-       * osxvideosink->superview. We now add osxwindow->gstview to the superview
-       * from the main thread
-       */
-      GST_INFO_OBJECT (osxvideosink, "we have a superview, adding our view to it");
-      gst_osx_video_sink_call_from_main_thread(osxvideosink, osxwindow->gstview,
-          @selector(addToSuperview:), osxvideosink->superview, NO);
-
-    } else {
-      if (osxvideosink->embed) {
-        /* the view wasn't added to a superview. It's possible that the
-         * application handled have-ns-view, stored our view internally and is
-         * going to add it to a superview later (webkit does that now).
-         */
-        GST_INFO_OBJECT (osxvideosink, "no superview");
-      } else {
-        gst_osx_video_sink_call_from_main_thread(osxvideosink,
-          osxvideosink->osxvideosinkobject,
-          @selector(createInternalWindow), nil, YES);
-        GST_INFO_OBJECT (osxvideosink, "No superview, creating an internal window.");
-      }
-    }
+  if (osxvideosink->superview == NULL) {
+    GST_INFO_OBJECT (osxvideosink, "emitting prepare-xwindow-id");
+    gst_video_overlay_prepare_window_handle (GST_VIDEO_OVERLAY (osxvideosink));
   }
+
+  if (osxvideosink->superview != NULL) {
+    /* prepare-xwindow-id was handled, we have the superview in
+     * osxvideosink->superview. We now add osxwindow->gstview to the superview
+     * from the main thread
+     */
+    GST_INFO_OBJECT (osxvideosink, "we have a superview, adding our view to it");
+    gst_osx_video_sink_call_from_main_thread (osxvideosink, osxwindow->gstview,
+        @selector(addToSuperview:), osxvideosink->superview, NO);
+
+  } else {
+    gst_osx_video_sink_call_from_main_thread (osxvideosink,
+      osxvideosink->osxvideosinkobject,
+      @selector(createInternalWindow), nil, YES);
+    GST_INFO_OBJECT (osxvideosink, "No superview, creating an internal window.");
+  }
+
+  gst_osx_video_sink_call_from_main_thread (osxvideosink, osxvideosink->osxvideosinkobject,
+    @selector(setActivationPolicy), nil, YES);
+
   [osxwindow->gstview setNavigation: GST_NAVIGATION(osxvideosink)];
   [osxvideosink->osxwindow->gstview setKeepAspectRatio: osxvideosink->keep_par];
 
@@ -330,10 +304,12 @@ gst_osx_video_sink_osxwindow_destroy (GstOSXVideoSink * osxvideosink)
   g_return_if_fail (GST_IS_OSX_VIDEO_SINK (osxvideosink));
   pool = [[NSAutoreleasePool alloc] init];
 
-  gst_osx_video_sink_call_from_main_thread(osxvideosink,
+  gst_osx_video_sink_call_from_main_thread (osxvideosink,
       osxvideosink->osxvideosinkobject,
       @selector(destroy), (id) nil, YES);
+#ifndef GSTREAMER_GLIB_COCOA_NSAPPLICATION
   gst_osx_video_sink_stop_cocoa_loop (osxvideosink);
+#endif
   [pool release];
 }
 
@@ -355,7 +331,7 @@ gst_osx_video_sink_osxwindow_resize (GstOSXVideoSink * osxvideosink,
 
   /* Directly resize the underlying view */
   GST_DEBUG_OBJECT (osxvideosink, "Calling setVideoSize on %p", osxwindow->gstview);
-  gst_osx_video_sink_call_from_main_thread(osxvideosink, object,
+  gst_osx_video_sink_call_from_main_thread (osxvideosink, object,
       @selector(resize), (id)nil, YES);
 
   [pool release];
@@ -389,6 +365,9 @@ gst_osx_video_sink_setcaps (GstBaseSink * bsink, GstCaps * caps)
 
   gst_osx_video_sink_osxwindow_resize (osxvideosink, osxvideosink->osxwindow,
       video_width, video_height);
+
+  gst_video_info_from_caps (&osxvideosink->info, caps);
+
   result = TRUE;
 
 beach:
@@ -405,7 +384,7 @@ gst_osx_video_sink_change_state (GstElement * element,
 
   osxvideosink = GST_OSX_VIDEO_SINK (element);
 
-  GST_DEBUG_OBJECT (osxvideosink, "%s => %s", 
+  GST_DEBUG_OBJECT (osxvideosink, "%s => %s",
         gst_element_state_get_name(GST_STATE_TRANSITION_CURRENT (transition)),
         gst_element_state_get_name(GST_STATE_TRANSITION_NEXT (transition)));
 
@@ -456,7 +435,7 @@ gst_osx_video_sink_show_frame (GstBaseSink * bsink, GstBuffer * buf)
 
   GST_DEBUG ("show_frame");
   bufferobject = [[GstBufferObject alloc] initWithBuffer:buf];
-  gst_osx_video_sink_call_from_main_thread(osxvideosink,
+  gst_osx_video_sink_call_from_main_thread (osxvideosink,
       osxvideosink->osxvideosinkobject,
       @selector(showFrame:), bufferobject, NO);
   [pool release];
@@ -485,9 +464,8 @@ gst_osx_video_sink_set_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case ARG_EMBED:
-      osxvideosink->embed = g_value_get_boolean(value);
       g_warning ("The \"embed\" property of osxvideosink is deprecated and "
-          "will be removed in the near future. Use the GstVideoOverlay "
+          "has no effect anymore. Use the GstVideoOverlay "
           "instead.");
       break;
     case ARG_FORCE_PAR:
@@ -514,7 +492,7 @@ gst_osx_video_sink_get_property (GObject * object, guint prop_id,
 
   switch (prop_id) {
     case ARG_EMBED:
-      g_value_set_boolean (value, osxvideosink->embed);
+      g_value_set_boolean (value, FALSE);
       break;
     case ARG_FORCE_PAR:
       g_value_set_boolean (value, osxvideosink->keep_par);
@@ -525,6 +503,14 @@ gst_osx_video_sink_get_property (GObject * object, guint prop_id,
   }
 }
 
+static gboolean
+gst_osx_video_sink_propose_allocation (GstBaseSink * base_sink, GstQuery * query)
+{
+    gst_query_add_allocation_meta (query,
+        GST_VIDEO_META_API_TYPE, NULL);
+
+    return TRUE;
+}
 
 static void
 gst_osx_video_sink_init (GstOSXVideoSink * sink)
@@ -540,12 +526,11 @@ gst_osx_video_sink_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_set_static_metadata (element_class, "OSX Video sink",
-      "Sink/Video", "OSX native videosink",
+  gst_element_class_set_static_metadata (element_class, "macOS Video sink",
+      "Sink/Video", "macOS native videosink",
       "Zaheer Abbas Merali <zaheerabbas at merali dot org>");
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_osx_video_sink_sink_template_factory));
+  gst_element_class_add_static_pad_template (element_class, &gst_osx_video_sink_sink_template_factory);
 }
 
 static void
@@ -586,26 +571,13 @@ gst_osx_video_sink_class_init (GstOSXVideoSinkClass * klass)
   gstbasesink_class->set_caps = gst_osx_video_sink_setcaps;
   gstbasesink_class->preroll = gst_osx_video_sink_show_frame;
   gstbasesink_class->render = gst_osx_video_sink_show_frame;
+  gstbasesink_class->propose_allocation = gst_osx_video_sink_propose_allocation;
   gstelement_class->change_state = gst_osx_video_sink_change_state;
 
-  /**
-   * GstOSXVideoSink:embed
-   *
-   * Set to #TRUE if you are embedding the video window in an application.
-   *
-   **/
-
   g_object_class_install_property (gobject_class, ARG_EMBED,
-      g_param_spec_boolean ("embed", "embed", "For ABI compatiblity only, do not use",
+      g_param_spec_boolean ("embed", "embed", "For ABI compatibility only, do not use",
           FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
-  /**
-   * GstOSXVideoSink:force-aspect-ratio
-   *
-   * When enabled, scaling will respect original aspect ratio.
-   *
-   **/
-
   g_object_class_install_property (gobject_class, ARG_FORCE_PAR,
       g_param_spec_boolean ("force-aspect-ratio", "force aspect ration",
           "When enabled, scaling will respect original aspect ration",
@@ -619,7 +591,9 @@ gst_osx_video_sink_navigation_send_event (GstNavigation * navigation,
   GstOSXVideoSink *osxvideosink = GST_OSX_VIDEO_SINK (navigation);
   GstPad *peer;
   GstEvent *event;
-  GstVideoRectangle src, dst, result;
+  GstVideoRectangle src = { 0, };
+  GstVideoRectangle dst = { 0, };
+  GstVideoRectangle result;
   NSRect bounds;
   gdouble x, y, xscale = 1.0, yscale = 1.0;
 
@@ -685,7 +659,7 @@ gst_osx_video_sink_set_window_handle (GstVideoOverlay * overlay, guintptr handle
   GstOSXVideoSink *osxvideosink = GST_OSX_VIDEO_SINK (overlay);
   NSView *view = (NSView *) handle_id;
 
-  gst_osx_video_sink_call_from_main_thread(osxvideosink,
+  gst_osx_video_sink_call_from_main_thread (osxvideosink,
       osxvideosink->osxvideosinkobject,
       @selector(setView:), view, YES);
 }
@@ -761,14 +735,14 @@ gst_osx_video_sink_get_type (void)
 
 - (void)windowWillClose:(NSNotification *)notification {
   /* Only handle close events if the window was closed manually by the user
-   * and not becuase of a state change state to READY */
+   * and not because of a state change state to READY */
   if (osxvideosink->osxwindow == NULL) {
     return;
   }
   if (!osxvideosink->osxwindow->closed) {
     osxvideosink->osxwindow->closed = TRUE;
     GST_ELEMENT_ERROR (osxvideosink, RESOURCE, NOT_FOUND, ("Output window was closed"), (NULL));
-    gst_osx_video_sink_osxwindow_destroy(osxvideosink);
+    gst_osx_video_sink_osxwindow_destroy (osxvideosink);
   }
 }
 
@@ -779,43 +753,36 @@ gst_osx_video_sink_get_type (void)
 -(id) initWithSink: (GstOSXVideoSink*) sink
 {
   self = [super init];
-  self->osxvideosink = sink;
+  self->osxvideosink = gst_object_ref (sink);
   return self;
 }
 
+-(void) dealloc {
+  gst_object_unref (osxvideosink);
+  [super dealloc];
+}
+
 -(void) createInternalWindow
 {
   GstOSXWindow *osxwindow = osxvideosink->osxwindow;
-  ProcessSerialNumber psn;
   NSRect rect;
   unsigned int mask;
 
+  [NSApplication sharedApplication];
+
   osxwindow->internal = TRUE;
 
-  mask =  NSTitledWindowMask             |
-          NSClosableWindowMask           |
-          NSResizableWindowMask          |
-          NSTexturedBackgroundWindowMask |
-          NSMiniaturizableWindowMask;
+  mask =  NSWindowStyleMaskTitled             |
+          NSWindowStyleMaskClosable           |
+          NSWindowStyleMaskResizable          |
+          NSWindowStyleMaskTexturedBackground |
+          NSWindowStyleMaskMiniaturizable;
 
   rect.origin.x = 100.0;
   rect.origin.y = 100.0;
   rect.size.width = (float) osxwindow->width;
   rect.size.height = (float) osxwindow->height;
 
-#ifndef RUN_NS_APP_THREAD
-  if (!osxvideosink->app_started) {
-    [NSApplication sharedApplication];
-    [NSApp finishLaunching];
-    osxvideosink->app_started = TRUE;
-  }
-#endif
-
-  if (!GetCurrentProcess(&psn)) {
-      TransformProcessType(&psn, kProcessTransformToForegroundApplication);
-      SetFrontProcess(&psn);
-  }
-
   osxwindow->win =[[[GstOSXVideoSinkWindow alloc]
                        initWithContentNSRect: rect
                        styleMask: mask
@@ -823,20 +790,23 @@ gst_osx_video_sink_get_type (void)
                        defer: NO
                        screen: nil] retain];
   GST_DEBUG("VideoSinkWindow created, %p", osxwindow->win);
-  [osxwindow->win makeKeyAndOrderFront:NSApp];
+  [osxwindow->win orderFrontRegardless];
   osxwindow->gstview =[osxwindow->win gstView];
   [osxwindow->win setDelegate:[[GstWindowDelegate alloc]
       initWithSink:osxvideosink]];
 
 }
 
-#ifdef RUN_NS_APP_THREAD
 + (BOOL) isMainThread
 {
   /* FIXME: ideally we should return YES only for ->ns_app_thread here */
   return YES;
 }
-#endif
+
+- (void) setActivationPolicy
+{
+  [NSApp setActivationPolicy: NSApplicationActivationPolicyRegular];
+}
 
 - (void) setView: (NSView*)view
 {
@@ -891,29 +861,56 @@ gst_osx_video_sink_get_type (void)
 
 - (void) showFrame: (GstBufferObject *) object
 {
-  GstMapInfo info;
-  guint8 *viewdata;
+  GstVideoFrame frame;
+  guint8 *readp, *writep;
+  gint i, active_width, stride;
+  guint8 *texture_buffer;
   NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
   GstBuffer *buf = object->buf;
 
-  if (osxvideosink->osxwindow != NULL)
-  {
-    gst_buffer_map (buf, &info, GST_MAP_READ);
-    viewdata = (guint8 *) [osxvideosink->osxwindow->gstview getTextureBuffer];
+  GST_OBJECT_LOCK (osxvideosink);
+  if (osxvideosink->osxwindow == NULL)
+      goto no_window;
 
-    if (G_UNLIKELY (viewdata == NULL)) {
-      GST_ELEMENT_ERROR (osxvideosink, RESOURCE, WRITE,
-        ("Could not get a texture buffer"), (NULL));
-    } else {
-      memcpy (viewdata, info.data, info.size);
-      [osxvideosink->osxwindow->gstview displayTexture];
-      gst_buffer_unmap (buf, &info);
-    }
+  texture_buffer = (guint8 *) [osxvideosink->osxwindow->gstview getTextureBuffer];
+  if (G_UNLIKELY (texture_buffer == NULL))
+      goto no_texture_buffer;
+
+  if (!gst_video_frame_map (&frame, &osxvideosink->info, buf, GST_MAP_READ))
+      goto no_map;
+
+  readp = GST_VIDEO_FRAME_PLANE_DATA (&frame, 0);
+  stride = GST_VIDEO_FRAME_PLANE_STRIDE (&frame, 0);
+  writep = texture_buffer;
+  active_width = GST_VIDEO_SINK_WIDTH (osxvideosink) * sizeof (short);
+  for (i = 0; i < GST_VIDEO_SINK_HEIGHT (osxvideosink); i++) {
+      memcpy (writep, readp, active_width);
+      writep += active_width;
+      readp += stride;
   }
+  [osxvideosink->osxwindow->gstview displayTexture];
 
+  gst_video_frame_unmap (&frame);
+
+out:
+  GST_OBJECT_UNLOCK (osxvideosink);
   [object release];
 
   [pool release];
+  return;
+
+no_map:
+  GST_WARNING_OBJECT (osxvideosink, "couldn't map frame");
+  goto out;
+
+no_window:
+  GST_WARNING_OBJECT (osxvideosink, "not showing frame since we have no window (!?)");
+  goto out;
+
+no_texture_buffer:
+  GST_ELEMENT_ERROR (osxvideosink, RESOURCE, WRITE, (NULL),
+          ("the texture buffer is NULL"));
+  goto out;
 }
 
 -(void) destroy
@@ -923,6 +920,7 @@ gst_osx_video_sink_get_type (void)
 
   pool = [[NSAutoreleasePool alloc] init];
 
+  GST_OBJECT_LOCK (osxvideosink);
   osxwindow = osxvideosink->osxwindow;
   osxvideosink->osxwindow = NULL;
 
@@ -940,10 +938,12 @@ gst_osx_video_sink_get_type (void)
     }
     g_free (osxwindow);
   }
+  GST_OBJECT_UNLOCK (osxvideosink);
+
   [pool release];
 }
 
-#ifdef RUN_NS_APP_THREAD
+#ifndef GSTREAMER_GLIB_COCOA_NSAPPLICATION
 -(void) nsAppThread
 {
   NSAutoreleasePool *pool;
@@ -973,7 +973,6 @@ gst_osx_video_sink_get_type (void)
 
   [pool release];
 }
-#endif
 
 -(void) checkMainRunLoop
 {
@@ -981,6 +980,7 @@ gst_osx_video_sink_get_type (void)
   g_cond_signal (&_run_loop_cond);
   g_mutex_unlock (&_run_loop_mutex);
 }
+#endif
 
 @end
 
@@ -1002,9 +1002,8 @@ gst_osx_video_sink_get_type (void)
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-
   if (!gst_element_register (plugin, "osxvideosink",
-          GST_RANK_PRIMARY, GST_TYPE_OSX_VIDEO_SINK))
+          GST_RANK_MARGINAL, GST_TYPE_OSX_VIDEO_SINK))
     return FALSE;
 
   GST_DEBUG_CATEGORY_INIT (gst_debug_osx_video_sink, "osxvideosink", 0,
