@@ -32,6 +32,7 @@
 
 #include "gstv4l2object.h"
 #include "v4l2-utils.h"
+#include "gstv4l2elements.h"
 
 #ifdef HAVE_GUDEV
 #include <gudev/gudev.h>
@@ -44,6 +45,8 @@ static GstV4l2Device *gst_v4l2_device_new (const gchar * device_path,
 
 G_DEFINE_TYPE (GstV4l2DeviceProvider, gst_v4l2_device_provider,
     GST_TYPE_DEVICE_PROVIDER);
+GST_DEVICE_PROVIDER_REGISTER_DEFINE (v4l2deviceprovider, "v4l2deviceprovider",
+    GST_RANK_PRIMARY, GST_TYPE_V4L2_DEVICE_PROVIDER);
 
 static void gst_v4l2_device_provider_finalize (GObject * object);
 static GList *gst_v4l2_device_provider_probe (GstDeviceProvider * provider);
@@ -366,6 +369,8 @@ provider_thread (gpointer data)
   if (context == NULL || loop == NULL) {
     provider->started = TRUE;
     g_cond_broadcast (&provider->started_cond);
+    g_clear_pointer (&loop, g_main_loop_unref);
+    g_clear_pointer (&context, g_main_context_unref);
     GST_OBJECT_UNLOCK (provider);
     return NULL;
   }
@@ -448,19 +453,24 @@ gst_v4l2_device_provider_stop (GstDeviceProvider * provider)
   self->loop = NULL;
   GST_OBJECT_UNLOCK (self);
 
-  if (!context || !loop)
+  if (!context || !loop) {
+    g_clear_pointer (&self->loop, g_main_loop_unref);
+    g_clear_pointer (&self->context, g_main_context_unref);
     return;
+  }
 
   idle_stop_source = g_idle_source_new ();
   g_source_set_callback (idle_stop_source, (GSourceFunc) g_main_loop_quit, loop,
-      (GDestroyNotify) g_main_loop_unref);
+      NULL);
   g_source_attach (idle_stop_source, context);
   g_source_unref (idle_stop_source);
-  g_main_context_unref (context);
 
   g_thread_join (self->thread);
   self->thread = NULL;
   self->started = FALSE;
+
+  g_main_loop_unref (loop);
+  g_main_context_unref (context);
 }
 
 #endif
