@@ -21,6 +21,7 @@
 
 /**
  * SECTION:element-waveformsink
+ * @title: waveformsink
  *
  * This element lets you output sound using the Windows WaveForm API.
  *
@@ -29,8 +30,7 @@
  * your pipeline works under all circumstances (those conversion elements will
  * act in passthrough-mode if no conversion is necessary).
  *
- * <refsect2>
- * <title>Example pipelines</title>
+ * ## Example pipelines
  * |[
  * gst-launch-1.0 -v audiotestsrc ! audioconvert ! volume volume=0.1 ! waveformsink
  * ]| will output a sine wave (continuous beep sound) to your sound card (with
@@ -38,7 +38,7 @@
  * |[
  * gst-launch-1.0 -v filesrc location=music.ogg ! decodebin ! audioconvert ! audioresample ! waveformsink
  * ]| will play an Ogg/Vorbis audio file and output it.
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -322,8 +322,8 @@ gst_waveform_sink_getcaps (GstBaseSink * bsink, GstCaps * filter)
     wfsink->cached_caps = gst_caps_ref (caps);
   }
 
-  GST_CAT_LOG_OBJECT (waveformsink_debug, wfsink, "Returning caps %s",
-      gst_caps_to_string (caps));
+  GST_CAT_LOG_OBJECT (waveformsink_debug, wfsink,
+      "Returning caps %" GST_PTR_FORMAT, caps);
 
   return caps;
 }
@@ -346,7 +346,7 @@ gst_waveform_sink_prepare (GstAudioSink * asink, GstAudioRingBufferSpec * spec)
   MMRESULT mmresult;
   guint index;
 
-  /* setup waveformex struture with the input ringbuffer specs */
+  /* setup waveformex structure with the input ringbuffer specs */
   memset (&wfx, 0, sizeof (wfx));
   wfx.cbSize = 0;
   wfx.wFormatTag = WAVE_FORMAT_PCM;
@@ -493,7 +493,7 @@ gst_waveform_sink_write (GstAudioSink * asink, gpointer data, guint length)
       if (mmresult != MMSYSERR_NOERROR) {
         waveOutGetErrorText (mmresult, wfsink->error_string, ERROR_LENGTH - 1);
         GST_CAT_WARNING_OBJECT (waveformsink_debug, wfsink,
-            "gst_waveform_sink_write: Error writting buffer to the device => %s",
+            "gst_waveform_sink_write: Error writing buffer to the device => %s",
             wfsink->error_string);
       }
 
@@ -506,7 +506,7 @@ gst_waveform_sink_write (GstAudioSink * asink, gpointer data, guint length)
       waveheader->dwUser = 0;
       wfsink->bytes_in_queue = 0;
       GST_CAT_LOG_OBJECT (waveformsink_debug, wfsink,
-          "gst_waveform_sink_write: Writting a buffer to the device (free buffers remaining=%d, write buffer=%d)",
+          "gst_waveform_sink_write: Writing a buffer to the device (free buffers remaining=%d, write buffer=%d)",
           wfsink->free_buffers_count, wfsink->write_buffer);
     }
   }
@@ -535,7 +535,7 @@ gst_waveform_sink_reset (GstAudioSink * asink)
   if (mmresult != MMSYSERR_NOERROR) {
     waveOutGetErrorText (mmresult, wfsink->error_string, ERROR_LENGTH - 1);
     GST_CAT_WARNING_OBJECT (waveformsink_debug, wfsink,
-        "gst_waveform_sink_reset: Error reseting waveform-audio device => %s",
+        "gst_waveform_sink_reset: Error resetting waveform-audio device => %s",
         wfsink->error_string);
   }
 }
