@@ -20,6 +20,7 @@
 
 /**
  * SECTION:element-ximagesrc
+ * @title: ximagesrc
  *
  * This element captures your X Display and creates raw RGB video.  It uses
  * the XDamage extension if available to only capture areas of the screen that
@@ -27,12 +28,11 @@
  * available to also capture your mouse pointer.  By default it will fixate to
  * 25 frames per second.
  *
- * <refsect2>
- * <title>Example pipelines</title>
+ * ## Example pipelines
  * |[
  * gst-launch-1.0 ximagesrc ! video/x-raw,framerate=5/1 ! videoconvert ! theoraenc ! oggmux ! filesink location=desktop.ogg
  * ]| Encodes your X display to an Ogg theora video at 5 frames per second.
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -47,7 +47,7 @@
 #include <X11/Xutil.h>
 
 #include <gst/gst.h>
-#include <gst/gst-i18n-plugin.h>
+#include <glib/gi18n-lib.h>
 #include <gst/video/video.h>
 
 #include "gst/glib-compat-private.h"
@@ -364,12 +364,23 @@ gst_ximage_src_unlock (GstBaseSrc * basesrc)
 static gboolean
 gst_ximage_src_recalc (GstXImageSrc * src)
 {
-  if (!src->xcontext)
-    return FALSE;
+  XWindowAttributes attrs;
+  gboolean has_changed = FALSE;
 
-  /* Maybe later we can check the display hasn't changed size */
-  /* We could use XQueryPointer to get only the current window. */
-  return TRUE;
+  g_assert (src->xwindow != 0 && src->xcontext);
+
+  if (XGetWindowAttributes (src->xcontext->disp, src->xwindow, &attrs)) {
+    if (src->xwin_width != attrs.width) {
+      src->xwin_width = attrs.width;
+      has_changed = TRUE;
+    }
+    if (src->xwin_height != attrs.height) {
+      src->xwin_height = attrs.height;
+      has_changed = TRUE;
+    }
+  }
+
+  return has_changed;
 }
 
 #ifdef HAVE_XFIXES
@@ -500,7 +511,7 @@ gst_ximage_src_ximage_get (GstXImageSrc * ximagesrc)
     ximagesrc->buffer_pool = g_slist_delete_link (ximagesrc->buffer_pool,
         ximagesrc->buffer_pool);
 
-    if ((meta->width == ximagesrc->width) ||
+    if ((meta->width == ximagesrc->width) &&
         (meta->height == ximagesrc->height))
       break;
 
@@ -547,9 +558,8 @@ gst_ximage_src_ximage_get (GstXImageSrc * ximagesrc)
     do {
       XDamageNotifyEvent *damage_ev = (XDamageNotifyEvent *) (&ev);
 
-      XNextEvent (ximagesrc->xcontext->disp, &ev);
-
-      if (ev.type == ximagesrc->damage_event_base + XDamageNotify &&
+      if (XCheckTypedEvent (ximagesrc->xcontext->disp,
+              ximagesrc->damage_event_base + XDamageNotify, &ev) &&
           damage_ev->level == XDamageReportNonEmpty) {
 
         XDamageSubtract (ximagesrc->xcontext->disp, ximagesrc->damage, None,
@@ -833,12 +843,6 @@ gst_ximage_src_create (GstPushSrc * bs, GstBuffer ** buf)
   GstClockTime dur;
   gint64 next_frame_no;
 
-  if (!gst_ximage_src_recalc (s)) {
-    GST_ELEMENT_ERROR (s, RESOURCE, FAILED,
-        (_("Changing resolution at runtime is not yet supported.")), (NULL));
-    return GST_FLOW_ERROR;
-  }
-
   if (s->fps_n <= 0 || s->fps_d <= 0)
     return GST_FLOW_NOT_NEGOTIATED;     /* FPS must be > 0 */
 
@@ -907,7 +911,19 @@ gst_ximage_src_create (GstPushSrc * bs, GstBuffer ** buf)
   s->last_frame_no = next_frame_no;
   GST_OBJECT_UNLOCK (s);
 
+  XGrabServer (s->xcontext->disp);
+
+  if (gst_ximage_src_recalc (s) && !gst_base_src_negotiate (GST_BASE_SRC (s))) {
+    XUngrabServer (s->xcontext->disp);
+    XSync (s->xcontext->disp, False);
+    return GST_FLOW_NOT_NEGOTIATED;
+  }
+
   image = gst_ximage_src_ximage_get (s);
+
+  XUngrabServer (s->xcontext->disp);
+  XSync (s->xcontext->disp, False);
+
   if (!image)
     return GST_FLOW_ERROR;
 
@@ -929,7 +945,7 @@ gst_ximage_src_set_property (GObject * object, guint prop_id,
     case PROP_DISPLAY_NAME:
 
       g_free (src->display_name);
-      src->display_name = g_strdup (g_value_get_string (value));
+      src->display_name = g_value_dup_string (value);
       break;
     case PROP_SHOW_POINTER:
       src->show_pointer = g_value_get_boolean (value);
@@ -945,9 +961,13 @@ gst_ximage_src_set_property (GObject * object, guint prop_id,
       break;
     case PROP_ENDX:
       src->endx = g_value_get_uint (value);
+      /* property comments say 0 means right/bottom, means we can't capture
+       * the top left pixel alone */
+      src->endx_fit_to_screen = (src->endx == 0);
       break;
     case PROP_ENDY:
       src->endy = g_value_get_uint (value);
+      src->endy_fit_to_screen = (src->endy == 0);
       break;
     case PROP_REMOTE:
       src->remote = g_value_get_boolean (value);
@@ -965,7 +985,7 @@ gst_ximage_src_set_property (GObject * object, guint prop_id,
         break;
       }
       g_free (src->xname);
-      src->xname = g_strdup (g_value_get_string (value));
+      src->xname = g_value_dup_string (value);
       break;
     default:
       break;
@@ -1070,32 +1090,21 @@ gst_ximage_src_get_caps (GstBaseSrc * bs, GstCaps * filter)
   if ((!s->xcontext) && (!gst_ximage_src_open_display (s, s->display_name)))
     return gst_pad_get_pad_template_caps (GST_BASE_SRC (s)->srcpad);
 
-  if (!gst_ximage_src_recalc (s))
-    return gst_pad_get_pad_template_caps (GST_BASE_SRC (s)->srcpad);
+  gst_ximage_src_recalc (s);
 
   xcontext = s->xcontext;
-  width = s->xcontext->width;
-  height = s->xcontext->height;
-  if (s->xwindow != 0) {
-    XWindowAttributes attrs;
-    int status = XGetWindowAttributes (s->xcontext->disp, s->xwindow, &attrs);
-    if (status) {
-      width = attrs.width;
-      height = attrs.height;
-    }
-  }
+  width = s->xwin_width;
+  height = s->xwin_height;
 
-  /* property comments say 0 means right/bottom, means we can't capture
-     the top left pixel alone */
-  if (s->endx == 0)
+  if (s->endx_fit_to_screen)
     s->endx = width - 1;
-  if (s->endy == 0)
+  if (s->endy_fit_to_screen)
     s->endy = height - 1;
 
   if (s->endx >= s->startx && s->endy >= s->starty) {
     /* this means user has put in values */
-    if (s->startx < xcontext->width && s->endx < xcontext->width &&
-        s->starty < xcontext->height && s->endy < xcontext->height) {
+    if (s->startx < width && s->endx < width &&
+        s->starty < height && s->endy < height) {
       /* values are fine */
       s->width = width = s->endx - s->startx + 1;
       s->height = height = s->endy - s->starty + 1;
@@ -1106,6 +1115,8 @@ gst_ximage_src_get_caps (GstBaseSrc * bs, GstCaps * filter)
       s->starty = 0;
       s->endx = width - 1;
       s->endy = height - 1;
+      s->endx_fit_to_screen = FALSE;
+      s->endy_fit_to_screen = FALSE;
     }
   } else {
     GST_WARNING ("User put in bogus co-ordinates, setting to full screen");
@@ -1113,6 +1124,8 @@ gst_ximage_src_get_caps (GstBaseSrc * bs, GstCaps * filter)
     s->starty = 0;
     s->endx = width - 1;
     s->endy = height - 1;
+    s->endx_fit_to_screen = FALSE;
+    s->endy_fit_to_screen = FALSE;
   }
   GST_DEBUG ("width = %d, height=%d", width, height);
 
@@ -1259,7 +1272,7 @@ gst_ximage_src_class_init (GstXImageSrcClass * klass)
    * known to work better with remote displays.
    */
   g_object_class_install_property (gc, PROP_REMOTE,
-      g_param_spec_boolean ("remote", "Remote dispay",
+      g_param_spec_boolean ("remote", "Remote display",
           "Whether the display is remote", FALSE,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
@@ -1312,6 +1325,8 @@ gst_ximage_src_init (GstXImageSrc * ximagesrc)
   ximagesrc->starty = 0;
   ximagesrc->endx = 0;
   ximagesrc->endy = 0;
+  ximagesrc->endx_fit_to_screen = TRUE;
+  ximagesrc->endy_fit_to_screen = TRUE;
   ximagesrc->remote = FALSE;
 }
 
