@@ -22,11 +22,12 @@
 
 #include "qtdemux.h"
 #include <glib/gprintf.h>
-#include <gst/check/gstharness.h>
-#include <gst/app/app.h>
 
 #include <gio/gio.h>
 
+#include <gst/check/check.h>
+#include <gst/app/app.h>
+
 #define TEST_FILE_PREFIX GST_TEST_FILES_PATH G_DIR_SEPARATOR_S
 
 static gboolean
