@@ -2,6 +2,7 @@
  * Copyright (C) 2006-2007 Tim-Philipp Müller <tim centricular net>
  * Copyright (C) 2008 Wouter Cloetens <wouter@mind.be>
  * Copyright (C) 2001-2003, Ximian, Inc.
+ * Copyright (C) 2021 Igalia S.L.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -28,15 +29,44 @@
 #include <glib.h>
 #include <glib/gprintf.h>
 
-#define SOUP_VERSION_MIN_REQUIRED (SOUP_VERSION_2_40)
 #include <libsoup/soup.h>
 #include <gst/check/gstcheck.h>
 
+#if ! SOUP_CHECK_VERSION(3, 0, 0)
 #if !defined(SOUP_MINOR_VERSION) || SOUP_MINOR_VERSION < 44
 #define SoupStatus SoupKnownStatusCode
 #endif
+#endif
 
-static guint http_port = 0, https_port = 0;
+#if SOUP_CHECK_VERSION(3, 0, 0)
+
+#define SOUP_AUTH_DOMAIN_BASIC_AUTH_CALLBACK "auth-callback"
+#define SOUP_AUTH_DOMAIN_DIGEST_AUTH_CALLBACK "auth-callback"
+
+#define gst_soup_uri_free g_uri_unref
+#define gst_soup_uri_to_string(x) g_uri_to_string_partial(x, G_URI_HIDE_PASSWORD)
+#define gst_soup_uri_get_port(x) g_uri_get_port(x)
+
+#else
+
+#define gst_soup_uri_free soup_uri_free
+#define gst_soup_uri_to_string(x) soup_uri_to_string(x, FALSE)
+#define gst_soup_uri_get_port(x) soup_uri_get_port(x)
+
+#define SoupServerMessage SoupMessage
+
+#define soup_server_message_get_method(x) (x->method)
+#define soup_server_message_get_http_version(x) soup_message_get_http_version(x)
+#define soup_server_message_get_status(x) (x->status_code)
+#define soup_server_message_set_status(x, s, r) soup_message_set_status(x, s)
+#define soup_server_message_get_reason_phrase(x) (x->reason_phrase)
+#define soup_server_message_get_uri(x) soup_message_get_uri(x)
+#define soup_server_message_get_request_headers(x) (x->request_headers)
+#define soup_server_message_get_response_headers(x) (x->response_headers)
+#define soup_server_message_get_request_body(x) (x->request_body)
+#define soup_server_message_get_response_body(x) (x->response_body)
+
+#endif
 
 gboolean redirect = TRUE;
 
@@ -53,8 +83,11 @@ static const char *realm = "SOUPHTTPSRC_REALM";
 static const char *basic_auth_path = "/basic_auth";
 static const char *digest_auth_path = "/digest_auth";
 
-static gboolean run_server (guint * http_port, guint * https_port);
-static void stop_server (void);
+static const char *ssl_cert_file = GST_TEST_FILES_PATH "/test-cert.pem";
+static const char *ssl_key_file = GST_TEST_FILES_PATH "/test-key.pem";
+
+static guint get_port_from_server (SoupServer * server);
+static SoupServer *run_server (gboolean use_https);
 
 static void
 handoff_cb (GstElement * fakesink, GstBuffer * buf, GstPad * pad,
@@ -85,22 +118,24 @@ digest_auth_cb (SoupAuthDomain * domain, SoupMessage * msg,
   return NULL;
 }
 
-static int
-run_test (const char *format, ...)
+static gboolean
+run_test (gboolean use_https, const gchar * path, gint expected)
 {
   GstStateChangeReturn ret;
-
   GstElement *pipe, *src, *sink;
-
   GstBuffer *buf = NULL;
-
   GstMessage *msg;
-
   gchar *url;
+  gboolean res = FALSE;
+  SoupServer *server;
+  guint port;
 
-  va_list args;
-
-  int rc = -1;
+  server = run_server (use_https);
+  if (server == NULL) {
+    g_print ("Failed to start up %s server", use_https ? "HTTPS" : "HTTP");
+    /* skip this test */
+    return TRUE;
+  }
 
   pipe = gst_pipeline_new (NULL);
 
@@ -114,19 +149,38 @@ run_test (const char *format, ...)
   gst_bin_add (GST_BIN (pipe), sink);
   fail_unless (gst_element_link (src, sink));
 
-  if (http_port == 0) {
-    GST_DEBUG ("failed to start soup http server");
-  }
-  fail_unless (http_port != 0);
-  va_start (args, format);
-  g_vasprintf (&url, format, args);
-  va_end (args);
+  port = get_port_from_server (server);
+  url = g_strdup_printf ("%s://127.0.0.1:%u%s",
+      use_https ? "https" : "http", port, path);
   fail_unless (url != NULL);
   g_object_set (src, "location", url, NULL);
   g_free (url);
 
+  if (use_https) {
+    GTlsDatabase *tlsdb;
+    GError *error = NULL;
+    gchar *path;
+
+    /* GTlsFileDatabase needs an absolute path. Using a relative one
+     * causes a warning from GLib-Net followed by a segfault in GnuTLS */
+    if (g_path_is_absolute (ssl_cert_file)) {
+      path = g_strdup (ssl_cert_file);
+    } else {
+      gchar *cwd = g_get_current_dir ();
+      path = g_build_filename (cwd, ssl_cert_file, NULL);
+      g_free (cwd);
+    }
+
+    tlsdb = g_tls_file_database_new (path, &error);
+    fail_unless (tlsdb, "Failed to load certificate: %s", error->message);
+
+    g_object_set (src, "tls-database", tlsdb, NULL);
+
+    g_object_unref (tlsdb);
+    g_free (path);
+  }
+
   g_object_set (src, "automatic-redirect", redirect, NULL);
-  g_object_set (src, "ssl-ca-file", GST_TEST_FILES_PATH "/test-cert.pem", NULL);
   if (cookies != NULL)
     g_object_set (src, "cookies", cookies, NULL);
   g_object_set (sink, "signal-handoffs", TRUE, NULL);
@@ -148,8 +202,8 @@ run_test (const char *format, ...)
       GST_MESSAGE_EOS | GST_MESSAGE_ERROR, -1);
   if (GST_MESSAGE_TYPE (msg) == GST_MESSAGE_ERROR) {
     gchar *debug = NULL;
-
     GError *err = NULL;
+    gint rc = -1;
 
     gst_message_parse_error (msg, &err, &debug);
     GST_INFO ("error: %s", err->message);
@@ -170,6 +224,8 @@ run_test (const char *format, ...)
     g_error_free (err);
     g_free (debug);
     gst_message_unref (msg);
+    GST_DEBUG ("Got HTTP error %u, expected %u", rc, expected);
+    res = (rc == expected);
     goto done;
   }
   gst_message_unref (msg);
@@ -183,6 +239,7 @@ run_test (const char *format, ...)
      * it might be for lots of reasons (no network connection, whatever), we're
      * not interested in those */
     GST_DEBUG ("didn't manage to get data within 10 seconds, skipping test");
+    res = TRUE;
     goto done;
   }
 
@@ -191,34 +248,34 @@ run_test (const char *format, ...)
   /* first buffer should have a 0 offset */
   fail_unless (GST_BUFFER_OFFSET (buf) == 0);
   gst_buffer_unref (buf);
-  rc = 0;
+  res = (expected == 0);
 
 done:
 
   gst_element_set_state (pipe, GST_STATE_NULL);
   gst_object_unref (pipe);
-  return rc;
+  gst_object_unref (server);
+  return res;
 }
 
 GST_START_TEST (test_first_buffer_has_offset)
 {
-  fail_unless (run_test ("http://127.0.0.1:%u/", http_port) == 0);
+  fail_unless (run_test (FALSE, "/", 0));
 }
 
 GST_END_TEST;
 
 GST_START_TEST (test_not_found)
 {
-  fail_unless (run_test ("http://127.0.0.1:%u/404", http_port) == 404);
-  fail_unless (run_test ("http://127.0.0.1:%u/404-with-data",
-          http_port) == 404);
+  fail_unless (run_test (FALSE, "/404", 404));
+  fail_unless (run_test (FALSE, "/404-with-data", 404));
 }
 
 GST_END_TEST;
 
 GST_START_TEST (test_forbidden)
 {
-  fail_unless (run_test ("http://127.0.0.1:%u/403", http_port) == 403);
+  fail_unless (run_test (FALSE, "/403", 403));
 }
 
 GST_END_TEST;
@@ -226,7 +283,7 @@ GST_END_TEST;
 GST_START_TEST (test_redirect_no)
 {
   redirect = FALSE;
-  fail_unless (run_test ("http://127.0.0.1:%u/302", http_port) == 302);
+  fail_unless (run_test (FALSE, "/302", 302));
 }
 
 GST_END_TEST;
@@ -234,17 +291,14 @@ GST_END_TEST;
 GST_START_TEST (test_redirect_yes)
 {
   redirect = TRUE;
-  fail_unless (run_test ("http://127.0.0.1:%u/302", http_port) == 0);
+  fail_unless (run_test (FALSE, "/302", 0));
 }
 
 GST_END_TEST;
 
 GST_START_TEST (test_https)
 {
-  if (!https_port)
-    GST_INFO ("Failed to start an HTTPS server; let's just skip this test.");
-  else
-    fail_unless (run_test ("https://127.0.0.1:%u/", https_port) == 0);
+  fail_unless (run_test (TRUE, "/", 0));
 }
 
 GST_END_TEST;
@@ -252,96 +306,96 @@ GST_END_TEST;
 GST_START_TEST (test_cookies)
 {
   static const char *biscotti[] = { "delacre=yummie", "koekje=lu", NULL };
-  int rc;
+  gboolean res;
 
   cookies = biscotti;
-  rc = run_test ("http://127.0.0.1:%u/", http_port);
+  res = run_test (FALSE, "/", 0);
   cookies = NULL;
-  fail_unless (rc == 0);
+  fail_unless (res);
 }
 
 GST_END_TEST;
 
 GST_START_TEST (test_good_user_basic_auth)
 {
-  int res;
+  gboolean res;
 
   user_id = good_user;
   user_pw = good_pw;
-  res = run_test ("http://127.0.0.1:%u%s", http_port, basic_auth_path);
+  res = run_test (FALSE, basic_auth_path, 0);
   GST_DEBUG ("Basic Auth user %s password %s res = %d", user_id, user_pw, res);
   user_id = user_pw = NULL;
-  fail_unless (res == 0);
+  fail_unless (res);
 }
 
 GST_END_TEST;
 
 GST_START_TEST (test_bad_user_basic_auth)
 {
-  int res;
+  gboolean res;
 
   user_id = bad_user;
   user_pw = good_pw;
-  res = run_test ("http://127.0.0.1:%u%s", http_port, basic_auth_path);
+  res = run_test (FALSE, basic_auth_path, 401);
   GST_DEBUG ("Basic Auth user %s password %s res = %d", user_id, user_pw, res);
   user_id = user_pw = NULL;
-  fail_unless (res == 401);
+  fail_unless (res);
 }
 
 GST_END_TEST;
 
 GST_START_TEST (test_bad_password_basic_auth)
 {
-  int res;
+  gboolean res;
 
   user_id = good_user;
   user_pw = bad_pw;
-  res = run_test ("http://127.0.0.1:%u%s", http_port, basic_auth_path);
+  res = run_test (FALSE, basic_auth_path, 401);
   GST_DEBUG ("Basic Auth user %s password %s res = %d", user_id, user_pw, res);
   user_id = user_pw = NULL;
-  fail_unless (res == 401);
+  fail_unless (res);
 }
 
 GST_END_TEST;
 
 GST_START_TEST (test_good_user_digest_auth)
 {
-  int res;
+  gboolean res;
 
   user_id = good_user;
   user_pw = good_pw;
-  res = run_test ("http://127.0.0.1:%u%s", http_port, digest_auth_path);
+  res = run_test (FALSE, digest_auth_path, 0);
   GST_DEBUG ("Digest Auth user %s password %s res = %d", user_id, user_pw, res);
   user_id = user_pw = NULL;
-  fail_unless (res == 0);
+  fail_unless (res);
 }
 
 GST_END_TEST;
 
 GST_START_TEST (test_bad_user_digest_auth)
 {
-  int res;
+  gboolean res;
 
   user_id = bad_user;
   user_pw = good_pw;
-  res = run_test ("http://127.0.0.1:%u%s", http_port, digest_auth_path);
+  res = run_test (FALSE, digest_auth_path, 401);
   GST_DEBUG ("Digest Auth user %s password %s res = %d", user_id, user_pw, res);
   user_id = user_pw = NULL;
-  fail_unless (res == 401);
+  fail_unless (res);
 }
 
 GST_END_TEST;
 
 GST_START_TEST (test_bad_password_digest_auth)
 {
-  int res;
+  gboolean res;
 
   user_id = good_user;
   user_pw = bad_pw;
-  res = run_test ("http://127.0.0.1:%u%s", http_port, digest_auth_path);
+  res = run_test (FALSE, digest_auth_path, 401);
   GST_DEBUG ("Digest Auth user %s password %s res = %d", user_id, user_pw, res);
   user_id = user_pw = NULL;
-  fail_unless (res == 401);
+  fail_unless (res);
 }
 
 GST_END_TEST;
@@ -389,7 +443,7 @@ GST_START_TEST (test_icy_stream)
 
   /* Radionomy Hot40Music shoutcast stream */
   g_object_set (src, "location",
-      "http://streaming.radionomy.com:80/Hot40Music", NULL);
+      "https://streaming.brol.tech/jamendolounge", NULL);
 
   /* EOS after the first buffer */
   g_object_set (src, "num-buffers", 1, NULL);
@@ -424,9 +478,6 @@ GST_START_TEST (test_icy_stream)
 
 GST_END_TEST;
 
-static SoupServer *server;      /* NULL */
-static SoupServer *ssl_server;  /* NULL */
-
 static Suite *
 souphttpsrc_suite (void)
 {
@@ -443,27 +494,19 @@ souphttpsrc_suite (void)
   tc_internet = tcase_create ("internet");
 
   suite_add_tcase (s, tc_chain);
-  if (run_server (&http_port, &https_port)) {
-    atexit (stop_server);
-    tcase_add_test (tc_chain, test_first_buffer_has_offset);
-    tcase_add_test (tc_chain, test_redirect_yes);
-    tcase_add_test (tc_chain, test_redirect_no);
-    tcase_add_test (tc_chain, test_not_found);
-    tcase_add_test (tc_chain, test_forbidden);
-    tcase_add_test (tc_chain, test_cookies);
-    tcase_add_test (tc_chain, test_good_user_basic_auth);
-    tcase_add_test (tc_chain, test_bad_user_basic_auth);
-    tcase_add_test (tc_chain, test_bad_password_basic_auth);
-    tcase_add_test (tc_chain, test_good_user_digest_auth);
-    tcase_add_test (tc_chain, test_bad_user_digest_auth);
-    tcase_add_test (tc_chain, test_bad_password_digest_auth);
-
-    if (ssl_server != NULL)
-      tcase_add_test (tc_chain, test_https);
-  } else {
-    g_print ("Skipping 12 souphttpsrc tests, couldn't start or connect to "
-        "local http server\n");
-  }
+  tcase_add_test (tc_chain, test_first_buffer_has_offset);
+  tcase_add_test (tc_chain, test_redirect_yes);
+  tcase_add_test (tc_chain, test_redirect_no);
+  tcase_add_test (tc_chain, test_not_found);
+  tcase_add_test (tc_chain, test_forbidden);
+  tcase_add_test (tc_chain, test_cookies);
+  tcase_add_test (tc_chain, test_good_user_basic_auth);
+  tcase_add_test (tc_chain, test_bad_user_basic_auth);
+  tcase_add_test (tc_chain, test_bad_password_basic_auth);
+  tcase_add_test (tc_chain, test_good_user_digest_auth);
+  tcase_add_test (tc_chain, test_bad_user_digest_auth);
+  tcase_add_test (tc_chain, test_bad_password_digest_auth);
+  tcase_add_test (tc_chain, test_https);
 
   suite_add_tcase (s, tc_internet);
   tcase_set_timeout (tc_internet, 250);
@@ -475,7 +518,7 @@ souphttpsrc_suite (void)
 GST_CHECK_MAIN (souphttpsrc);
 
 static void
-do_get (SoupMessage * msg, const char *path)
+do_get (SoupServerMessage * msg, const char *path)
 {
   gboolean send_error_doc = FALSE;
   char *uri;
@@ -484,7 +527,7 @@ do_get (SoupMessage * msg, const char *path)
 
   SoupStatus status = SOUP_STATUS_OK;
 
-  uri = soup_uri_to_string (soup_message_get_uri (msg), FALSE);
+  uri = gst_soup_uri_to_string (soup_server_message_get_uri (msg));
   GST_DEBUG ("request: \"%s\"", uri);
 
   if (!strcmp (path, "/301"))
@@ -506,20 +549,21 @@ do_get (SoupMessage * msg, const char *path)
     char *redir_uri;
 
     redir_uri = g_strdup_printf ("%s-redirected", uri);
-    soup_message_headers_append (msg->response_headers, "Location", redir_uri);
+    soup_message_headers_append (soup_server_message_get_response_headers (msg),
+        "Location", redir_uri);
     g_free (redir_uri);
   }
   if (status != (SoupStatus) SOUP_STATUS_OK && !send_error_doc)
     goto leave;
 
-  if (msg->method == SOUP_METHOD_GET) {
+  if (soup_server_message_get_method (msg) == SOUP_METHOD_GET) {
     char *buf;
 
     buf = g_malloc (buflen);
     memset (buf, 0, buflen);
-    soup_message_body_append (msg->response_body, SOUP_MEMORY_TAKE,
-        buf, buflen);
-  } else {                      /* msg->method == SOUP_METHOD_HEAD */
+    soup_message_body_append (soup_server_message_get_response_body (msg),
+        SOUP_MEMORY_TAKE, buf, buflen);
+  } else {                      /* method == SOUP_METHOD_HEAD */
 
     char *length;
 
@@ -528,13 +572,13 @@ do_get (SoupMessage * msg, const char *path)
      * malloc.
      */
     length = g_strdup_printf ("%lu", (gulong) buflen);
-    soup_message_headers_append (msg->response_headers,
+    soup_message_headers_append (soup_server_message_get_response_headers (msg),
         "Content-Length", length);
     g_free (length);
   }
 
 leave:
-  soup_message_set_status (msg, status);
+  soup_server_message_set_status (msg, status, NULL);
   g_free (uri);
 }
 
@@ -545,22 +589,29 @@ print_header (const char *name, const char *value, gpointer data)
 }
 
 static void
-server_callback (SoupServer * server, SoupMessage * msg,
+server_callback (SoupServer * server, SoupServerMessage * msg,
     const char *path, GHashTable * query,
-    SoupClientContext * context, gpointer data)
+#if !SOUP_CHECK_VERSION(3, 0, 0)
+    SoupClientContext * context,
+#endif
+    gpointer data)
 {
-  GST_DEBUG ("%s %s HTTP/1.%d", msg->method, path,
-      soup_message_get_http_version (msg));
-  soup_message_headers_foreach (msg->request_headers, print_header, NULL);
-  if (msg->request_body->length)
-    GST_DEBUG ("%s", msg->request_body->data);
+  const char *method = soup_server_message_get_method (msg);
 
-  if (msg->method == SOUP_METHOD_GET || msg->method == SOUP_METHOD_HEAD)
+  GST_DEBUG ("%s %s HTTP/1.%d", method, path,
+      soup_server_message_get_http_version (msg));
+  soup_message_headers_foreach (soup_server_message_get_request_headers (msg),
+      print_header, NULL);
+  if (soup_server_message_get_request_body (msg)->length)
+    GST_DEBUG ("%s", soup_server_message_get_request_body (msg)->data);
+
+  if (method == SOUP_METHOD_GET || method == SOUP_METHOD_HEAD)
     do_get (msg, path);
   else
-    soup_message_set_status (msg, SOUP_STATUS_NOT_IMPLEMENTED);
+    soup_server_message_set_status (msg, SOUP_STATUS_NOT_IMPLEMENTED, NULL);
 
-  GST_DEBUG ("  -> %d %s", msg->status_code, msg->reason_phrase);
+  GST_DEBUG ("  -> %d %s", soup_server_message_get_status (msg),
+      soup_server_message_get_reason_phrase (msg));
 }
 
 static guint
@@ -571,90 +622,94 @@ get_port_from_server (SoupServer * server)
 
   uris = soup_server_get_uris (server);
   g_assert (g_slist_length (uris) == 1);
-  port = soup_uri_get_port (uris->data);
-  g_slist_free_full (uris, (GDestroyNotify) soup_uri_free);
+  port = gst_soup_uri_get_port (uris->data);
+  g_slist_free_full (uris, (GDestroyNotify) gst_soup_uri_free);
 
   return port;
 }
 
-static gboolean
-run_server (guint * http_port, guint * https_port)
+static SoupServer *
+run_server (gboolean use_https)
 {
-  guint port = SOUP_ADDRESS_ANY_PORT;
-  guint ssl_port = SOUP_ADDRESS_ANY_PORT;
-  const char *ssl_cert_file = GST_TEST_FILES_PATH "/test-cert.pem";
-  const char *ssl_key_file = GST_TEST_FILES_PATH "/test-key.pem";
-  static int server_running = 0;
-  GSocketAddress *address;
-  GError *err = NULL;
+  SoupServer *server = soup_server_new (NULL, NULL);
+  SoupServerListenOptions listen_flags = 0;
+  guint port;
 
-  SoupAuthDomain *domain = NULL;
 
-  if (server_running)
-    return TRUE;
-
-  server_running = 1;
-
-  *http_port = *https_port = 0;
-
-  server = soup_server_new (NULL, NULL);
-  if (!server) {
-    GST_DEBUG ("Unable to create server");
-    return FALSE;
-  }
-  soup_server_add_handler (server, NULL, server_callback, NULL, NULL);
-  domain = soup_auth_domain_basic_new (SOUP_AUTH_DOMAIN_REALM, realm,
-      SOUP_AUTH_DOMAIN_BASIC_AUTH_CALLBACK, basic_auth_cb,
-      SOUP_AUTH_DOMAIN_ADD_PATH, basic_auth_path, NULL);
-  soup_server_add_auth_domain (server, domain);
-  g_object_unref (domain);
-  domain = soup_auth_domain_digest_new (SOUP_AUTH_DOMAIN_REALM, realm,
-      SOUP_AUTH_DOMAIN_DIGEST_AUTH_CALLBACK, digest_auth_cb,
-      SOUP_AUTH_DOMAIN_ADD_PATH, digest_auth_path, NULL);
-  soup_server_add_auth_domain (server, domain);
-  g_object_unref (domain);
-
-  address = g_inet_socket_address_new_from_string ("0.0.0.0", port);
-  soup_server_listen (server, address, 0, &err);
-  g_object_unref (address);
-  if (err) {
-    stop_server ();
-    g_clear_error (&err);
-    return FALSE;
-  }
-
-  *http_port = get_port_from_server (server);
-  GST_DEBUG ("HTTP server listening on port %u", *http_port);
-
-  if (ssl_cert_file && ssl_key_file) {
+  if (use_https) {
     GTlsBackend *backend = g_tls_backend_get_default ();
+    GError *err = NULL;
 
-    if (backend != NULL && g_tls_backend_supports_tls (backend)) {
-      ssl_server = soup_server_new (SOUP_SERVER_SSL_CERT_FILE, ssl_cert_file,
-          SOUP_SERVER_SSL_KEY_FILE, ssl_key_file, NULL);
-    } else {
+    if (backend == NULL || !g_tls_backend_supports_tls (backend)) {
       GST_INFO ("No TLS support");
+      g_object_unref (server);
+      return NULL;
     }
-
-    if (ssl_server) {
-      GST_INFO ("HTTPS server listening on port %u", *https_port);
-      soup_server_add_handler (ssl_server, NULL, server_callback, NULL, NULL);
-      address = g_inet_socket_address_new_from_string ("0.0.0.0", ssl_port);
-      soup_server_listen (ssl_server, address, SOUP_SERVER_LISTEN_HTTPS, &err);
-      g_object_unref (address);
-
-      if (err) {
-        GST_ERROR ("Failed to start HTTPS server: %s", err->message);
-        stop_server ();
-        g_clear_error (&err);
-        return FALSE;
+#if SOUP_CHECK_VERSION(3, 0, 0)
+    {
+      GTlsCertificate *cert =
+          g_tls_certificate_new_from_files (ssl_cert_file, ssl_key_file,
+          &err);
+      if (!cert) {
+        GST_INFO ("Failed to load certificate: %s", err->message);
+        g_error_free (err);
+        return NULL;
       }
+      soup_server_set_tls_certificate (server, cert);
+      g_object_unref (cert);
+    }
+#else
+    if (!soup_server_set_ssl_cert_file (server, ssl_cert_file, ssl_key_file,
+            &err)) {
+      GST_INFO ("Failed to load certificate: %s", err->message);
+      g_object_unref (server);
+      g_error_free (err);
+      return NULL;
+    }
+#endif
 
-      *https_port = get_port_from_server (ssl_server);
-      GST_DEBUG ("HTTPS server listening on port %u", *https_port);
+    listen_flags |= SOUP_SERVER_LISTEN_HTTPS;
+  }
+
+  soup_server_add_handler (server, NULL, server_callback, NULL, NULL);
+
+  {
+    SoupAuthDomain *domain;
+
+    domain = soup_auth_domain_basic_new ("realm", realm,
+        SOUP_AUTH_DOMAIN_BASIC_AUTH_CALLBACK, basic_auth_cb, NULL);
+    soup_auth_domain_add_path (domain, basic_auth_path);
+    soup_server_add_auth_domain (server, domain);
+    g_object_unref (domain);
+
+    domain = soup_auth_domain_digest_new ("realm", realm,
+        SOUP_AUTH_DOMAIN_DIGEST_AUTH_CALLBACK, digest_auth_cb, NULL);
+    soup_auth_domain_add_path (domain, digest_auth_path);
+    soup_server_add_auth_domain (server, domain);
+    g_object_unref (domain);
+  }
+
+  {
+    GSocketAddress *address;
+    GError *err = NULL;
+
+    address = g_inet_socket_address_new_from_string ("0.0.0.0", 0);
+    soup_server_listen (server, address, listen_flags, &err);
+    g_object_unref (address);
+
+    if (err) {
+      GST_ERROR ("Failed to start %s server: %s",
+          use_https ? "HTTPS" : "HTTP", err->message);
+      g_object_unref (server);
+      g_error_free (err);
+      return NULL;
     }
   }
 
+  port = get_port_from_server (server);
+  GST_DEBUG ("%s server listening on port %u", use_https ? "HTTPS" : "HTTP",
+      port);
+
   /* check if we can connect to our local http server */
   {
     GSocketConnection *conn;
@@ -662,48 +717,18 @@ run_server (guint * http_port, guint * https_port)
 
     client = g_socket_client_new ();
     g_socket_client_set_timeout (client, 2);
-    conn = g_socket_client_connect_to_host (client, "127.0.0.1", *http_port,
-        NULL, NULL);
+    conn =
+        g_socket_client_connect_to_host (client, "127.0.0.1", port, NULL, NULL);
     if (conn == NULL) {
-      GST_INFO ("Couldn't connect to http server 127.0.0.1:%u", *http_port);
+      GST_INFO ("Couldn't connect to 127.0.0.1:%u", port);
       g_object_unref (client);
-      stop_server ();
-      return FALSE;
+      g_object_unref (server);
+      return NULL;
     }
+
     g_object_unref (conn);
-
-    if (ssl_server == NULL)
-      goto skip_https_check;
-
-    conn = g_socket_client_connect_to_host (client, "127.0.0.1", *https_port,
-        NULL, NULL);
-    if (conn == NULL) {
-      GST_INFO ("Couldn't connect to https server 127.0.0.1:%u", *https_port);
-      g_object_unref (client);
-      stop_server ();
-      return FALSE;
-    }
-    g_object_unref (conn);
-
-  skip_https_check:
-
     g_object_unref (client);
   }
 
-  return TRUE;
-}
-
-static void
-stop_server (void)
-{
-  GST_INFO ("cleaning up");
-
-  if (server) {
-    g_object_unref (server);
-    server = NULL;
-  }
-  if (ssl_server) {
-    g_object_unref (ssl_server);
-    ssl_server = NULL;
-  }
+  return server;
 }
